/*
 *  Copyright 2002-2005, Mersad Team, Allameh Helli High School (NODET).
 *
 *  This program is free software, you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Library General Public License for more details.
 *
 *  This file is created by: Mohammad Salehe
 *  and is modified by: Meisam Vosoughpour
 *
 *  Released on Monday 1 August 2005, 10 Mordad 1384 by Mersad RoboCup Team.
 *  For more information please read README file.
*/

#include <cmath>

#include <Clear.h>
#include <Basics.h>
#include <Logger.h>
#include <BasicKick.h>
#include <SayDecision.h>

#include <AdvancedAgent.h>

using namespace std;

#define CLEAR_TO_PLAYER_DELTA_X 8

#define CLEAR_LOG1

// class Clear
Clear::Clear(const WorldModel *worldModel):
		AdvancedAction(worldModel)
{
}

void Clear::execute(Form &form, const Library &library)
{
	LOG << "Clear::execute" << endl;
	command = clearCommand;

	form.sayForm.beforePassSay = true;
	form.sayForm.interceptNum = twoKickableInterceptNum;
}

float Clear::getValue(const Library &library)
{
	Point kickPoint;
	Vector goalVector;
	int i;
	
	twoKickableInterceptNum = 0;

	float defaultClearWeight = 60;
	LOG << "Clear::getValue" << endl;

	if (!worldModel->isBallKickable())
		return AD_MIN_VALUE;

	if (worldModel->getBallStatus() == BS_KICKABLE_OPP ||
		worldModel->getBallStatus() == BS_KICKABLE_BODY_OPP)
	{
		if (worldModel->getBall().getDistance(Point(52.5, 0)) < 16)
		{
#ifdef CLEAR_LOG1
	LOG << "Clear::TwoKickable model 1" << endl;
#endif
			bool up;
			const Player *player = worldModel->getOppGoalie();
			if (player == NULL)
			{
				if (worldModel->getBall().getPos().getY() > 0)
					up = true;
				else
					up = false;
			}
			else if (player->getPos().getY() > 0)
				up = false;
			else
				up = true;

			if (up)
			{
				Point gccGhat(52.5, 5.5);
				clearCommand = FastKickToPoint(AT_CLEAR, gccGhat,
					worldModel->getBall(), worldModel->getBody()).getCommand();
			}
			else
			{
				Point gccGhat(52.5, -5.5);
				clearCommand = FastKickToPoint(AT_CLEAR, gccGhat,
					worldModel->getBall(), worldModel->getBody()).getCommand();
			}
		}
		else
		{
#ifdef CLEAR_LOG1
	LOG << "Clear::TwoKickable model 2" << endl;
#endif
			bool up;
			const Player *player = worldModel->getOppGoalie();
			Point clearPoints[20];
			int clearPointsNum = 0;
			unsigned clearPlayers[20];
			float clearWeights[20];
			float teammateDistance;
			float distanceWeight, offenseWeight;
			float oppBallRate;
			float oppBallRateMin;

			float maxWeight;
			int maxPointNum;

			oppBallRateMin = 100;
			
			for (i = 0; i < FULL_PLAYERS_NUM; i++)
				if (worldModel->getFullPlayer(TID_OPPONENT, i).isValid())
				{
					oppBallRate = (worldModel->getFullPlayer(TID_OPPONENT, i).
						getDistance(worldModel->getBall()) - worldModel->
						getFullPlayer(TID_OPPONENT, i).getSize() - 
						worldModel->getServerParam()["ball_size"].asFloat()) /
						worldModel->getFullPlayer(TID_OPPONENT, i).getKickableMargin();
					if (oppBallRate <= 1)
					{
						if (oppBallRate < oppBallRateMin)
							oppBallRateMin = oppBallRate;
					}
				}

			for (i = 0; i < HALF_PLAYERS_NUM; i++)
				if (worldModel->getHalfPlayer(TID_OPPONENT, i).isValid())
				{
					oppBallRate = (worldModel->getHalfPlayer(TID_OPPONENT, i).
						getDistance(worldModel->getBall()) - worldModel->
						getHalfPlayer(TID_OPPONENT, i).getSize() - 
						worldModel->getServerParam()["ball_size"].asFloat()) /
						worldModel->getHalfPlayer(TID_OPPONENT, i).getKickableMargin();
					if (oppBallRate <= 1)
					{
						if (oppBallRate < oppBallRateMin)
							oppBallRateMin = oppBallRate;
					}
				}
			
			if (oppBallRateMin > 10)
				oppBallRateMin = 0.1;
			else
				oppBallRateMin = 1 - oppBallRateMin;

#ifdef CLEAR_LOG1
	LOG << "Clear:oppBallRateMin(multiplier)= " << oppBallRateMin << endl;
#endif
					
			for (i = 0; i < FULL_PLAYERS_NUM; i++)
				if (worldModel->getFullPlayer(TID_TEAMMATE, i).isValid())
				{
					if (i + 1 == GOALIE_UNIFORM_NUM)
						continue;
					clearPoints[clearPointsNum] = worldModel->getFullPlayer(TID_TEAMMATE, i).
						getPos();
					clearPoints[clearPointsNum].x += CLEAR_TO_PLAYER_DELTA_X * oppBallRateMin;
					clearPlayers[clearPointsNum] = i + 1;
					teammateDistance = worldModel->getFullPlayer(TID_TEAMMATE, i).
						getBodyVec().getMagnitude();
					if (teammateDistance < 2)
						distanceWeight = 10;
					else if (teammateDistance >= 2 && teammateDistance < 20)
						distanceWeight = Basics::reRate(teammateDistance,
								2, 20, 10, 100);
					else
						distanceWeight = 9;
					offenseWeight = Basics::reRate(worldModel->getFullPlayer(TID_TEAMMATE, i).
						getPos().getX() - worldModel->getBody().getPos().getX(), -10, 18,
						0, 100);
					clearWeights[clearPointsNum] = distanceWeight * 0.5 + offenseWeight * 1.1;
#ifdef CLEAR_LOG1
	LOG << "Clear:addPlayer " << i + 1 << " weight=" <<
		clearWeights[clearPointsNum] << " dw,ow=" << distanceWeight << " " <<
		offenseWeight << endl;
#endif
					if (!FastKickToPoint(AT_CLEAR, clearPoints[clearPointsNum],
						worldModel->getBall(), worldModel->getBody()).isPossible())
					{
#ifdef CLEAR_LOG1
	LOG << "Clear:not possible " << endl; 
#endif
						clearWeights[clearPointsNum] = AD_MIN_VALUE;
					}
					clearPointsNum++;
				}

			// Clear to opponent goal
			if (worldModel->getBall().getDistance(Point(52.5, 0)) < 16)
			{
				if (player == NULL)
				{
					if (worldModel->getBall().getPos().getY() > 0)
						up = true;
					else
						up = false;
				}
				else if (player->getPos().getY() > 0)
					up = false;
				else
					up = true;

				if (up)
				{
					clearPoints[clearPointsNum] = Point(52.5, 10);
				}
				else
				{
					clearPoints[clearPointsNum] = Point(52.5, -10);
				}
				clearPlayers[clearPointsNum] = 0;
				clearWeights[clearPointsNum] = 90 + (worldModel->getBody().getPos().getX() - 20);
#ifdef CLEAR_LOG1
	LOG << "Clear:addGoal " << (up ? "up" : "down") << " weight=" <<
#endif
				clearWeights[clearPointsNum] << endl;
				if (!FastKickToPoint(AT_CLEAR, clearPoints[clearPointsNum],
					worldModel->getBall(), worldModel->getBody()).isPossible())
				{
#ifdef CLEAR_LOG1
	LOG << "Clear:not possible " << endl; 
#endif
					clearWeights[clearPointsNum] = AD_MIN_VALUE;
				}
				clearPointsNum++;
			}
			
			clearPoints[clearPointsNum] = Point(52.5, worldModel->getBody().getPos().getX());
			clearPlayers[clearPointsNum] = 0;
			clearWeights[clearPointsNum] = 1;
#ifdef CLEAR_LOG1
	LOG << "Clear:addDummy " << " weight=" << clearWeights[clearPointsNum] << endl;
#endif
			if (!FastKickToPoint(AT_CLEAR, clearPoints[clearPointsNum],
				worldModel->getBall(), worldModel->getBody()).isPossible())
			{
#ifdef CLEAR_LOG1
	LOG << "Clear:not possible " << endl; 
#endif
				clearWeights[clearPointsNum] = AD_MIN_VALUE;
			}
			clearPointsNum++;
		
			maxPointNum = 0;
			maxWeight = -0xFFFF * 10;

			for (i = 0; i < clearPointsNum; i++)
				if (clearWeights[i] > maxWeight)
				{
					maxWeight = clearWeights[i];
					maxPointNum = i;
				}

#ifdef CLEAR_LOG1
	LOG << "Clear::selected =" << maxPointNum << " " << clearPoints[maxPointNum] << endl;
	LOG << "Clear::selected player=" << clearPlayers[maxPointNum] << endl;
#endif
			if (maxWeight <= AD_MIN_VALUE)
			{
#ifdef CLEAR_LOG1
	LOG << "Clear::all not possible =" << endl;
#endif
				if (worldModel->getBody().getPos().getY() >= 0)
					clearCommand = new KickCommand(AT_CLEAR, 100, 50);
				else
					clearCommand = new KickCommand(AT_CLEAR, 100, -50);
			}
			else
			{	
				clearCommand = FastKickToPoint(AT_CLEAR, clearPoints[maxPointNum],
					worldModel->getBall(), worldModel->getBody()).getCommand();
				twoKickableInterceptNum = clearPlayers[maxPointNum];
			}
		}

		return AD_ALWAYS_RUN_VALUE;
	}
	else
	{ // Defense kick
#ifdef CLEAR_LOG1
	LOG << "Clear::defense" << endl;
#endif
		if ((worldModel->getBall().getPos().getX() >= -28 ||
			fabs(worldModel->getBall().getPos().getY()) >= 21) &&
			!worldModel->getBody().isGoalie())
			return AD_MIN_VALUE;

		Point clearPoints[20];
		int clearPointsNum = 0;
		float clearWeights[20];
		bool clearToTeammate[20];
		Ball clearBall;
		Command *curClearCommand;

		float maxWeight;
		int maxPointNum;
		InterceptCalculate clearIntercept;
		float interceptX;

		Point oppPoint;
		
		clearPoints[0] = Point(0, 20);
		clearPoints[1] = Point(0, 10);
		clearPoints[2] = Point(0, 0);
		clearPoints[3] = Point(0, -10);
		clearPoints[4] = Point(0, -20);
		clearPoints[5] = Point(-14, 30);
		clearPoints[6] = Point(-14, -30);
		clearPoints[7] = Point(0, worldModel->getBody().getPos().getY());
		if (worldModel->getBody().getPos().getY() > 0)
			clearPoints[8] = Point(worldModel->getBody().getPos().getX(), 30);
		else
			clearPoints[8] = Point(worldModel->getBody().getPos().getX(), -30);
		clearPointsNum = 9;

		clearIntercept.startSession(1);
		clearIntercept.updateByWorldModel(worldModel);
		
		clearIntercept.setTmmSlowDashPower(20);
		clearIntercept.setTmmFastDashPower(80);
		clearIntercept.setTmmDirectDashPower(100);
		clearIntercept.setTmmSeeBallDelay(2);
		clearIntercept.setTmmPosDevRate(0.8);

		clearIntercept.setOppSlowDashPower(20);
		clearIntercept.setOppFastDashPower(90);
		clearIntercept.setOppDirectDashPower(100);
		clearIntercept.setOppSeeBallDelay(1);
		clearIntercept.setOppPosDevRate(0.8);
		
		for (i = 0; i < clearPointsNum; i++)
		{
			FastKickToPoint fastKick(AT_CLEAR, clearPoints[i], worldModel->getBall(),
					worldModel->getBody());
			if (!fastKick.isPossible())
			{
#ifdef CLEAR_LOG1
	LOG << "Clear:point [" << i << "] = " << clearPoints[i] << " not possible" << endl;
#endif
				clearWeights[i] = AD_MIN_VALUE;
				if (fastKick.getCommand())
					delete fastKick.getCommand();
				continue;
			}
			curClearCommand = fastKick.getCommand();
			clearBall = worldModel->getBall();
			clearBall.simulateByAction(worldModel->getBody(), curClearCommand);
			
			
			clearIntercept.setVirtualBall(clearBall);
			clearIntercept.setKickCycles(1);
			clearIntercept.run();

#ifdef CLEAR_LOG1
	LOG << "Clear:point [" << i << "] = " << clearPoints[i] << endl;
#endif

			if (clearIntercept.existFastestPlayer())
			{
				if (clearIntercept.getFastestPlayer().getTeamId() == TID_TEAMMATE)
				{
#ifdef CLEAR_LOG1
	LOG << "Clear:fastest exist teammate" << endl;
#endif
					interceptX = clearIntercept.getFastestInterPlayer().getInterPoint().x;
#ifdef CLEAR_LOG1
	LOG << "Clear:interceptX = " << interceptX << endl;
#endif
					if (interceptX < -36)
						clearWeights[i] = -10;
					else if (interceptX < -25)
						clearWeights[i] = Basics::reRate(interceptX, -36, -25, -10, 45);
					else
						clearWeights[i] = Basics::reRate(interceptX, -25, 10, 45, 70);
					clearWeights[i] += 25;
					clearToTeammate[i] = true;
				}
				else
				{
#ifdef CLEAR_LOG1
	LOG << "Clear:fastest exist opponent" << endl;
#endif
					interceptX = clearIntercept.getFastestInterPlayer().getInterPoint().x;
#ifdef CLEAR_LOG1
	LOG << "Clear:interceptX = " << interceptX << endl;
#endif
					if (interceptX < -36)
						clearWeights[i] = -50;
					else if (interceptX < -25)
						clearWeights[i] = Basics::reRate(interceptX, -36, -25, -50, 5);
					else
						clearWeights[i] = Basics::reRate(interceptX, -25, 10, 5, 25);
					clearToTeammate[i] = false;
				}
			}
			else
			{
				clearWeights[i] = 20;
			}
#ifdef CLEAR_LOG1
	LOG << "Clear:clearWeight = " << clearWeights[i] << endl;
#endif
		}

		maxPointNum = 0;
		maxWeight = -0xFFFF * 10;

		for (i = 0; i < clearPointsNum; i++)
			if (clearWeights[i] > maxWeight)
			{
				maxWeight = clearWeights[i];
				maxPointNum = i;
			}

		if (maxWeight <= AD_MIN_VALUE)
		{
#ifdef CLEAR_LOG1
	LOG << "Clear: all not possible" << endl;
#endif
			if (worldModel->getBody().getPos().getY() >= 0)
				clearCommand = new KickCommand(AT_CLEAR, 100, 50);
			else
				clearCommand = new KickCommand(AT_CLEAR, 100, -50);
			return -60;
		}
#ifdef CLEAR_LOG1
	LOG << "Clear::selected =" << maxPointNum << " " << clearPoints[maxPointNum] << endl;
	LOG << "Clear::weight =" << maxWeight << endl;
#endif

		clearCommand = FastKickToPoint(AT_CLEAR, clearPoints[maxPointNum],
			worldModel->getBall(), worldModel->getBody()).getCommand();

		if (clearToTeammate[maxPointNum])
			maxWeight -= 15;

#ifdef CLEAR_LOG1
	LOG << "Clear::weight(after add1) =" << maxWeight << endl;
#endif
			
		maxWeight += Basics::reRate(worldModel->getBall().getPos().getX(),
				-53, -34, 2, 20, RRM_REVERSE);
		
#ifdef CLEAR_LOG1
	LOG << "Clear::weight(after add2) =" << maxWeight << endl;
#endif

		for (i = 0; i < FULL_PLAYERS_NUM; i++)
			if (worldModel->getFullPlayer(TID_OPPONENT, i).isValid())
			{
				oppPoint = worldModel->getFullPlayer(TID_OPPONENT, i).getPos();
				if (oppPoint.x > worldModel->getBody().getPos().getX() - 2.5 &&
					oppPoint.x < worldModel->getBody().getPos().getX() + 5 &&
					oppPoint.y > worldModel->getBody().getPos().getY() - 7 &&
					oppPoint.y < worldModel->getBody().getPos().getY() + 7)
				{
					maxWeight += Basics::reRate(worldModel->getBody().getDistance(oppPoint),
							0, 9, 1, 4, RRM_REVERSE);
#ifdef CLEAR_LOG1
	LOG << "Clear::addOpp " << i + 1 << endl;
#endif
				}
			}

#ifdef CLEAR_LOG1
	LOG << "Clear::weight(after add3) =" << maxWeight << endl;
#endif

		if (worldModel->getBody().isGoalie())
			return AD_ALWAYS_RUN_VALUE;

		if (maxWeight > 75)
			return AD_ALWAYS_RUN_VALUE;
		else
			return maxWeight;

		// Previous Methdod
		if ((worldModel->getBall().getPos().getX() >= -36 ||
			fabs(worldModel->getBall().getPos().getY()) >= 20))
			return AD_MIN_VALUE;
			
		if(worldModel->getBall().getVel().getMagnitude() > 2)
		{
			if(worldModel->getBall().getAbsVec().getMagnitude() > .8)
			{
				if(worldModel->getBody().getPos().getY() < 0)
				{
					kickPoint.x = 0.00; ////////////////TOF Chasbide
					kickPoint.y = 9.00;
					clearCommand = FastKickToPoint(AT_CLEAR, kickPoint, worldModel->getBall(),
						worldModel->getBody()).getCommand();
					
					if ((worldModel->getBall().getPos().getX() >= -46 ||
						fabs(worldModel->getBall().getPos().getY()) >= 15))
						return defaultClearWeight;
					else
						return AD_ALWAYS_RUN_VALUE;
				}
				else
				{
					kickPoint.x = 0.00; ////////////////TOF Chasbide
					kickPoint.y = -9.00;
					clearCommand = FastKickToPoint(AT_CLEAR, kickPoint, worldModel->getBall(),
						worldModel->getBody()).getCommand();
					if ((worldModel->getBall().getPos().getX() >= -46 ||
						fabs(worldModel->getBall().getPos().getY()) >= 15))
						return defaultClearWeight;
					else
						return AD_ALWAYS_RUN_VALUE;
				}	
			}
			else
			{
				kickPoint.x = worldModel->getBody().getPos().getX() + 42; ////////////////TOF Chasbide
				kickPoint.y = worldModel->getBody().getPos().getY();
				clearCommand = FastKickToPoint(AT_CLEAR, kickPoint, worldModel->getBall(),
					worldModel->getBody()).getCommand();
				if ((worldModel->getBall().getPos().getX() >= -46 ||
					fabs(worldModel->getBall().getPos().getY()) >= 15))
					return defaultClearWeight;
				else
					return AD_ALWAYS_RUN_VALUE;
			}
		}
		else if(worldModel->getBall().getVel().getMagnitude() > .9)
		{
			Vector ballVector;
			ballVector.setByPoints(worldModel->getBody().getPos(), 
				worldModel->getBall().getPos());
				
			ballVector.setAsPolar(.45, ballVector.getDirection());
			
			kickPoint.x = worldModel->getBody().getPos().getX() + ballVector.getX();
			kickPoint.y = worldModel->getBody().getPos().getY() + ballVector.getY();
			
			clearCommand = KickToPointArriveVelocity(AT_CLEAR, kickPoint, 0, 
				worldModel->getBall(), worldModel->getBody()).getCommand();

			if ((worldModel->getBall().getPos().getX() >= -46 ||
				fabs(worldModel->getBall().getPos().getY()) >= 15))
				return defaultClearWeight;
			else
				return AD_ALWAYS_RUN_VALUE;
		}	
		else
		{
			///in ke joloye darvaze dar mavaghei, bayad be darvazeban pass bede va hamin joori be jolo shoot nakone ra bayad ba'dan bezaram
			kickPoint.x = worldModel->getBody().getPos().getX() + 42; ////////////////TOF Chasbide
			kickPoint.y = worldModel->getBody().getPos().getY();
			clearCommand = KickToPointArriveVelocity(AT_CLEAR, kickPoint, 0, 
				worldModel->getBall(), worldModel->getBody()).getCommand();
			if ((worldModel->getBall().getPos().getX() >= -46 ||
				fabs(worldModel->getBall().getPos().getY()) >= 15))
				return defaultClearWeight;
			else
				return AD_ALWAYS_RUN_VALUE;
		}
	}
	return AD_MIN_VALUE;
}

