/*
 *  Copyright 2002-2005, Mersad Team, Allameh Helli High School (NODET).
 *
 *  This program is free software, you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Library General Public License for more details.
 *
 *  This file is created by: Meisam Vosoughpour
 *
 *  Released on Monday 1 August 2005, 10 Mordad 1384 by Mersad RoboCup Team.
 *  For more information please read README file.
*/

#include <cmath>

#include <Logger.h>
#include <BasicDash.h>
#include <BasicTurn.h>
#include <Positioning.h>
#include <Basics.h>
#include <cmath>


using namespace std;

Positioning::Positioning(const WorldModel *worldModel):
		worldModel(worldModel)
{
	coorseVar = false;
	setSimplePos();
}

void Positioning::setSimplePos()
{
	attraction[0] = .35;
	ballPos[0] = Point(-52.5, 0);
	playerPos[0][4] = Point(-49, 9);
	playerPos[0][2] = Point(-49, -3);
	playerPos[0][3] = Point(-49, 3);
	playerPos[0][1] = Point(-49, -9);
	playerPos[0][5] = Point(-34, 17);
	playerPos[0][7] = Point(-44, 0);
	playerPos[0][6] = Point(-34, -17);
	playerPos[0][8] = Point(-13, 0);
	playerPos[0][9] = Point(-10, -20);
	playerPos[0][10] = Point(-10, 20);



	attraction[1] = .35;
	ballPos[1] = Point(-23, 0);
	playerPos[1][4] = Point(-30, 15);
	playerPos[1][2] = Point(-30, -5);
	playerPos[1][3] = Point(-30, 5);
	playerPos[1][1] = Point(-30, -15);
	playerPos[1][5] = Point(-20, 10);
	playerPos[1][7] = Point(-25, 0);
	playerPos[1][6] = Point(-20, -10);
	playerPos[1][8] = Point(0, 0);
	playerPos[1][9] = Point(0, -18);
	playerPos[1][10] = Point(0, 18);

	attraction[3] = .35;
	ballPos[3] = Point(17, 0);
	playerPos[3][4] = Point(14, 16);
	playerPos[3][2] = Point(14, -5.8);
	playerPos[3][3] = Point(14, 5.8);
	playerPos[3][1] = Point(14, -16);
	playerPos[3][5] = Point(20, 10);
	playerPos[3][7] = Point(15, 0);
	playerPos[3][6] = Point(20, -10);
	playerPos[3][8] = Point(24, 0);
	playerPos[3][9] = Point(23, -20);
	playerPos[3][10] = Point(23, 20);


	attraction[2] = .35;
	ballPos[2] = Point(0, 0);

	for (int i = 1; i < 11; i++)
	{	
	
		playerPos[2][i].x = playerPos[1][i].x + (((ballPos[2].x - ballPos[1].x) / 
				(ballPos[3].x - ballPos[1].x)) * 
				(playerPos[3][i].x - playerPos[1][i].x));

		playerPos[2][i].y = playerPos[1][i].y + (((ballPos[2].x - ballPos[1].x) / 
				(ballPos[3].x - ballPos[1].x)) * 
				(playerPos[3][i].y - playerPos[1][i].y));
	}



	attraction[7] = .35;
	ballPos[7] = Point(52.5, 0);
	playerPos[7][4] = Point(10, 15);
	playerPos[7][2] = Point(0, -10);
	playerPos[7][3] = Point(0, 10);
	playerPos[7][1] = Point(10, -15);
	playerPos[7][5] = Point(50, 15);
	playerPos[7][7] = Point(17, 0);
	playerPos[7][6] = Point(50, -15);
	playerPos[7][8] = Point(49, 0);
	playerPos[7][9] = Point(48, -(+40));
	playerPos[7][10] = Point(48, -(-40));

	attraction[4] = .35;
	ballPos[4] = Point(24, 0);

	for (int i = 1; i < 11; i++)
	{	
		playerPos[4][i].x = playerPos[3][i].x + (((ballPos[4].x - ballPos[3].x) / 
				(ballPos[7].x - ballPos[3].x)) * 
				(playerPos[7][i].x - playerPos[3][i].x));

		playerPos[4][i].y = playerPos[3][i].y + (((ballPos[4].x - ballPos[3].x) / 
				(ballPos[7].x - ballPos[3].x)) * 
				(playerPos[7][i].y - playerPos[3][i].y));
	}

	attraction[5] = .4;
	ballPos[5] = Point(52.5, 0);
	playerPos[5][4] = Point(10, 15);
	playerPos[5][2] = Point(0, -10);
	playerPos[5][3] = Point(0, 10);
	playerPos[5][1] = Point(10, -15);
	playerPos[5][5] = Point(35, 10);
	playerPos[5][7] = Point(17, 0);
	playerPos[5][6] = Point(35, -10);
	playerPos[5][8] = Point(51, 0);
	playerPos[5][9] = Point(50, -3);
	playerPos[5][10] = Point(50, 3);
}

void Positioning::setHomePoints()
{
	int region;
	for (region = 0; region < 6; region++) 
	{		
		if (worldModel->getBall().getPos().getX() < ballPos[0].x)
		{
			region = 0;
			break;
		}
		if (worldModel->getBall().getPos().getX() > ballPos[4].x)
		{
			region = 4;
			break;
		}
		if (worldModel->getBall().getPos().getX() < ballPos[region + 1].x)
			break;
	}

	float attract = .3;

	for (int i = 1; i < 11; i++)
	{
	LOG << "MEISAM2005 Positioning aaaaah : REGION :  " << region << endl;
		basicPos[i].x = playerPos[region][i].x + (((worldModel->getBall().getPos().getX() - 
			ballPos[region].x) / (ballPos[region + 1].x - ballPos[region].x)) * 
			(playerPos[region + 1][i].x - playerPos[region][i].x));				
				
		basicPos[i].y = playerPos[region][i].y + (((worldModel->getBall().getPos().getX() - 
			ballPos[region].x) / (ballPos[region + 1].x - ballPos[region].x)) * 
			(playerPos[region + 1][i].y - playerPos[region][i].y));				
		
		attract = attraction[region] + (((worldModel->getBall().getPos().getX() - 
			ballPos[region].x) / (ballPos[region + 1].x - ballPos[region].x)) * 
			(attraction[region + 1] - attraction[region]));

//		if ((i == 3 || i == 2 || (i == 1 && worldModel->getBall().getPos().getY() > 0)
//		 		|| (i == 4 && worldModel->getBall().getPos().getY() < 0)
//				|| i == 7) && 
//				worldModel->getBall().getPos().getX() < 60)
//			attract = .05;	


/////////////////PLAYER 9&10&11 BEG  BUUUUUUUUUUUUUGGGG LFOKJGJ
//	HHHHHHHHHHHHEEEEEEEEEEEEEEEEEEEELLLLLLLLLLLLLLLLLLLLPPPPPPPPPPPPPPP
		if (i == 9 && worldModel->getBall().getPos().getX() > 40
				&& fabs(worldModel->getBall().getPos().getY()) < 15)
			attract = .1;	
		if (i == 10 && worldModel->getBall().getPos().getX() > 40
				&& fabs(worldModel->getBall().getPos().getY()) < 15)
			attract = .1;	
		if (i == 11 && worldModel->getBall().getPos().getX() > 40
				&& fabs(worldModel->getBall().getPos().getY()) < 15)
			attract = .1;	
/////////////////PLAYER 9&10&11 END


		
		if (i == 8 && basicPos[i].x > 44.5)
			basicPos[i].x = 44.5; 

///////////////////////OFFENSE SECTION
		

		if (i == 5 && worldModel->getBall().getPos().getY() < -5 &&
			worldModel->getBall().getPos().getX() > 35)
		{	
			attract = 2;	
			basicPos[i].x -= 0;
			if (basicPos[i].x > 39)
				basicPos[i].x = 39;
		}

		if (i == 5 && worldModel->getBall().getPos().getY() < -5 &&
			worldModel->getBall().getPos().getX() > 45)
		{	
			attract = 2;	
			basicPos[i].x -= 0;
			if (basicPos[i].x > 40)
				basicPos[i].x = 40;
		}
		
		if (i == 6 && worldModel->getBall().getPos().getY() > 5 &&
			worldModel->getBall().getPos().getX() > 35)
		{	
			attract = 2;	
			basicPos[i].x -= 0;
			if (basicPos[i].x > 39)
				basicPos[i].x = 39;
		}

		if (i == 6 && worldModel->getBall().getPos().getY() > 5 &&
			worldModel->getBall().getPos().getX() > 45)
		{	
			attract = 2;	
			basicPos[i].x -= 0;
			if (basicPos[i].x > 40)
				basicPos[i].x = 40;
		}
			
///////////////////////OFFENSE SECTION

		basicPos[i].y += worldModel->getBall().getPos().getY() * attract;
		
		if ((i == 2 || i == 3) && basicPos[i].x > -1)
			basicPos[i].x = -1;
		
		if ((i == 1 || i == 4) && basicPos[i].x > -1 && 
			worldModel->getBall().getPos().getX() < 17)
			basicPos[i].x = -1;

		if ((i == 1 || i == 4) && basicPos[i].x > 5) 
			basicPos[i].x = 4;

//ghame ma'navi	  if (i == 7 && basicPos[i].x > 4 && 
//ghame ma'navi			worldModel->getBall().getPos().getX() < 25)
//ghame ma'navi			basicPos[i].x = 4;

//ghame ma'navi		if (i == 7 && basicPos[i].x > -1 && 
//ghame ma'navi			worldModel->getBall().getPos().getX() < 17)
//ghame ma'navi			basicPos[i].x = -1;

//ghame ma'navi		if (i == 7 && basicPos[i].x > 8)
//ghame ma'navi			basicPos[i].x = 8;
////////////////////THISO IFS
		if ((i <= 4 || i == 7) && worldModel->getBall().getPos().getX() > -28
			&& worldModel->getBall().getPos().getX() < 0)
		{
//			float unitStep = 6;
//			float deffStep = ((int)(worldModel->getBall().getPos().getX() / 
//						unitStep) - 1) * unitStep;
//			basicPos[i].x = deffStep;
		}
		
		if (i == 3)
			LOG << "MEISAM2005 positioning: basicPos[3] A : " << basicPos[i] << endl;

		if (i == 2 || i == 3)
		{
			float maxXFor34 = getMaxXFor34();
			if (worldModel->getBall().getPos().getX() > -38)
			{
				if (basicPos[i].x > maxXFor34 + ENSEJAM_MAX_X) 

					basicPos[i].x = maxXFor34 + ENSEJAM_MAX_X;
			}
			else
			{
				if (basicPos[i].x > maxXFor34 + 7)
					basicPos[i].x = maxXFor34 + 7;
			}	
			LOG << "maxXFor3&4 : " << maxXFor34 << endl;
		}
		if (i == 7 && worldModel->getBall().getPos().getX() > -38)
		{
//ghame ma'navi			float maxXFor7 = getMaxXFor7();
//ghame ma'navi			float extraFor7 = 3.6;
//ghame ma'navi			if (basicPos[i].x > maxXFor7 + extraFor7)
//ghame ma'navi				basicPos[i].x = maxXFor7 + extraFor7;
//ghame ma'navi			LOG << "maxXFor7 : " << maxXFor7 << endl;
		}
////////////////////THISO IFS


			 	
		float offsideLine = worldModel->getOppOffsideLine();
		LOG << "oppOffsideLine : " << offsideLine << endl;
		
		offsideLine -= 3.5;
				
		if ((i == 9 || i == 10)	&& 
			worldModel->getBall().getPos().getX() > 38)
		{
			basicPos[i].x = fmax(worldModel->getBall().getPos().getX(), 
								basicPos[i].x);
			basicPos[i].x = fmin(basicPos[i].x, 50);					
		}	 

		
		if (i == 10 && basicPos[i].y < -2)
			basicPos[i].y = -2;	
		if (i == 9 && basicPos[i].y > 2)
			basicPos[i].y = 2;
			
		if (basicPos[i].y > 32)
			basicPos[i].y = 32;
		if (basicPos[i].y < -32)
			basicPos[i].y = -32;
////////////////Akharin rooz : begin
		if (i == 9 && worldModel->getBall().getPos().getX() > 36 &&
			worldModel->getBall().getPos().getY() < -7)
		{
			float extraX;
			if (worldModel->getBall().getPos().getY() < -28)
				extraX = 7;
			else	
				extraX = Basics::reRate(
					fabs(worldModel->getBall().getPos().getY()), 7, 28, 0, 7); 
			basicPos[i].x -= extraX;
			if (basicPos[i].x < 36)
				basicPos[i].x = 36;
			basicPos[i].y -= 1.5;	
				
		}		
		
		if (i == 10 && worldModel->getBall().getPos().getX() > 36 &&
			worldModel->getBall().getPos().getY() > 7)
		{
			float extraX;
			if (worldModel->getBall().getPos().getY() > 28)
				extraX = 7;
			else	
				extraX = Basics::reRate(
					fabs(worldModel->getBall().getPos().getY()), 7, 28, 0, 7); 
			basicPos[i].x -= extraX;
			if (basicPos[i].x < 36)
				basicPos[i].x = 36;
			basicPos[i].y += 1.5;	
		}		
		if (i == 8 && worldModel->getBall().getPos().getX() > 40 &&
			fabs(worldModel->getBall().getPos().getY()) > 7)
		{
			float extraX;
			if (fabs(worldModel->getBall().getPos().getY()) > 25)
				extraX = 3;
			else	
				extraX = Basics::reRate(
						fabs(worldModel->getBall().getPos().getY()), 7, 25, 0, 3);
			basicPos[i].x -= extraX;
			if (basicPos[i].x < 36)
				basicPos[i].x = 36;
		}		
		
		if (i == 9 && worldModel->getBall().getPos().getX() > 36 &&
			worldModel->getBall().getPos().getY() > 5)
		{
			float extraX;
			if (worldModel->getBall().getPos().getY() > 15)
				extraX = 0;
			else
				extraX = Basics::reRate(
					fabs(worldModel->getBall().getPos().getY()), 5, 15, 0, 0);
			basicPos[i].x += extraX;
			if (basicPos[i].x > 51)
				basicPos[i].x = 51;
		}		
		if (i == 10 && worldModel->getBall().getPos().getX() > 36 &&
			worldModel->getBall().getPos().getY() < -5)
		{
			float extraX;
			if (worldModel->getBall().getPos().getY() < -15)
				extraX = 0;
			else
				extraX = Basics::reRate(
					fabs(worldModel->getBall().getPos().getY()), 5, 15, 0, 0); 
			basicPos[i].x += extraX;
			if (basicPos[i].x > 51)
				basicPos[i].x = 51;
		}		
////////////////Akharin rooz : end		

///////////////////////OFFENSE SECTION
		if (i == 5 && basicPos[i].y < 0 &&
 			worldModel->getBall().getPos().getX() > 35)
			basicPos[i].y = 0;

		if (i == 6 && basicPos[i].y > 0 &&
 			worldModel->getBall().getPos().getX() > 35)
			basicPos[i].y = 0;
///////////////////////OFFENSE SECTION

		if (i == 1 || i == 2 || i == 3 || i == 4)
			if (basicPos[i].x > -1)
				basicPos[i].x = -1; 
		
//Meisam2005
	if (i == 1 && worldModel->getBall().getPos().getX() > 15)
	{
		basicPos[i].x += worldModel->getBall().getPos().getX() * .2;
		if (worldModel->getBall().getPos().getY() >= 0)
			basicPos[i].x -= fabs(worldModel->getBall().getPos().getY()) * .2;
		if (worldModel->getBall().getPos().getY() < 0)
			basicPos[i].x += fabs(worldModel->getBall().getPos().getY()) * .2;
		if (basicPos[i].x < -1)
			basicPos[i].x = -1;
		LOG << "Meisam2005 Positioning :  basicPos[1]" << basicPos[i].x << endl; 	
	}

	if (i == 4 && worldModel->getBall().getPos().getX() > 15)
	{
		basicPos[i].x += worldModel->getBall().getPos().getX() * .2;
		if (worldModel->getBall().getPos().getY() < 0)
			basicPos[i].x -= fabs(worldModel->getBall().getPos().getY()) * .2;
		if (worldModel->getBall().getPos().getY() >= 0) 
			basicPos[i].x += fabs(worldModel->getBall().getPos().getY()) * .2;
		if (basicPos[i].x < -1)
			basicPos[i].x = -1;
		LOG << "Meisam2005 Positioning :  basicPos[4]" << basicPos[i].x << endl; 	
	}
//Meisam2005
	
	if (i == 3)
		LOG << "MEISAM2005 positioning: basicPos[3] B : " << basicPos[i] << endl;

	if (i == 8 && worldModel->getBall().getPos().getX() < 36)
	{
		basicPos[i].y += min(worldModel->getBall().getPos().getY() * .3 , 6.00);
		basicPos[i].y = min(basicPos[10].y - 7 , basicPos[i].y);
		basicPos[i].y = max(basicPos[9].y + 7 , basicPos[i].y);
		
	}

	if (i >= 8)
	{
		if (coorseVar == false)
		{
			if (basicPos[i].x - worldModel->getBody().getPos().getX() > 8 &&
				worldModel->getBall().getPos().getX() > 0 &&
				abs(worldModel->getBody().getPos().getY() - 
					basicPos[i].y) < 8)
			{
				coorseVar = true;
				basicPos[i].y = worldModel->getBody().getPos().getY();
				LOG << "Meisam2005 coorseVar 1 " << endl;
			}		
		}
		else
		{
			if (basicPos[i].x - worldModel->getBody().getPos().getX() > 6 &&
				worldModel->getBall().getPos().getX() > 0 &&
				abs(worldModel->getBody().getPos().getY() - 
					basicPos[i].y) < 10)
			{
				coorseVar = true;
				basicPos[i].y = worldModel->getBody().getPos().getY();
				LOG << "Meisam2005 coorseVar 2 " << endl;
			}		
		}
	}


	if (i == 5 || i == 6)
	{
		if (coorseVar == false)
		{
			if (basicPos[i].x - worldModel->getBody().getPos().getX() > 6 &&
				worldModel->getBall().getPos().getX() > 0 &&
				abs(worldModel->getBody().getPos().getY() - 
					basicPos[i].y) < 8)
			{
				coorseVar = true;
				basicPos[i].y = worldModel->getBody().getPos().getY();
				LOG << "Meisam2005 coorseVar 1 " << endl;
			}		
		}
		else
		{
			if (basicPos[i].x - worldModel->getBody().getPos().getX() > 4 &&
				worldModel->getBall().getPos().getX() > 0 &&
				abs(worldModel->getBody().getPos().getY() - 
					basicPos[i].y) < 10)
			{
				coorseVar = true;
				basicPos[i].y = worldModel->getBody().getPos().getY();
				LOG << "Meisam2005 coorseVar 2 " << endl;
			}		
		}
	}

		if (basicPos[i].x > offsideLine && 
				(worldModel->getPlayMode() == PM_PLAY_ON))
			basicPos[i].x = offsideLine;
	}
}

Command *Positioning::getPositioningCommand(Point homePoint) const
{
	float maxDashPower = 100;		

	// Stamina Manager
/*	if (worldModel->getBody().getPostNum() == 5 || 
				worldModel->getBody().getPostNum() == 2)  
	{
		maxDashPower = 100;

		if (worldModel->getBody().getPos().getX() < homePoint.x && 
				homePoint.x > -36)
		{
			maxDashPower = 50;		
			if (worldModel->getBody().getStamina() < 2300)
				maxDashPower = worldModel->getBody().getStaminaIncMax();
		}
				
		if (worldModel->getBall().getPos().getX() > 0 && 
			!((worldModel->getBody().getPos().getX() - homePoint.x) > 4) && 
			worldModel->getBody().getStamina() < 3000)
			maxDashPower = worldModel->getBody().getStaminaIncMax() / 2; 
				
		if (worldModel->getBall().getPos().getX() > 15 && 
			!((worldModel->getBody().getPos().getX() - homePoint.x) > 6) && 
			worldModel->getBody().getStamina() < 3000)
			maxDashPower = worldModel->getBody().getStaminaIncMax() / 3; 
	}*/
	
	if (worldModel->getBody().getPostNum() == 4 || 
		worldModel->getBody().getPostNum() == 3 ||  
		worldModel->getBody().getPostNum() == 2 || 
		worldModel->getBody().getPostNum() == 5)  
{
		maxDashPower = 100;

		if (worldModel->getBody().getPos().getX() < homePoint.x && 
			homePoint.x > -36 && worldModel->getBall().getPos().getX() < 0)
		{
			maxDashPower = 60;		
			if (worldModel->getBody().getStamina() < 2300)
				maxDashPower = worldModel->getBody().getStaminaIncMax();
		}

		if (worldModel->getBall().getPos().getX() > 0 && 
			!((worldModel->getBody().getPos().getX() - homePoint.x) > 4) && 
			worldModel->getBody().getStamina() < 2800)
			maxDashPower = worldModel->getBody().getStaminaIncMax() / 2; 

		if (worldModel->getBall().getPos().getX() > 15 && 
			!((worldModel->getBody().getPos().getX() - homePoint.x) > 6) && 
			worldModel->getBody().getStamina() < 2800)
			maxDashPower = worldModel->getBody().getStaminaIncMax() / 3; 
	}
		
	if (worldModel->getBody().getPostNum() == 8 || 
				worldModel->getBody().getPostNum() == 7)
	{
		maxDashPower = 85;
		if (worldModel->getBody().getStamina() < 2300)
			maxDashPower = worldModel->getBody().getStaminaIncMax();
		if (worldModel->getBody().getStamina() < 2000)
			maxDashPower = worldModel->getBody().getStaminaIncMax() / 2;
	}
		
	if (worldModel->getBody().getPostNum() == 11 || 
		worldModel->getBody().getPostNum() == 9 || 
		worldModel->getBody().getPostNum() == 10 || 
		worldModel->getBody().getPostNum() == 6)  
	{
		maxDashPower = 100;
		if (worldModel->getBody().getPos().getX() > homePoint.x)
		{
			if (worldModel->getBall().getPos().getX() < 20)
			{
				maxDashPower = 80;		
				if (worldModel->getBody().getStamina() < 1700)
					maxDashPower = worldModel->getBody().getStaminaIncMax();
				if (worldModel->getBody().getStamina() < 1600)
					maxDashPower = worldModel->getBody().getStaminaIncMax() / 2;
			}

			if (worldModel->getBall().getPos().getX() < 5)
			{
				maxDashPower = 65;		
				if (worldModel->getBody().getStamina() < 2300)
					maxDashPower = worldModel->getBody().getStaminaIncMax();
				if (worldModel->getBody().getStamina() < 2000)
					maxDashPower = worldModel->getBody().getStaminaIncMax() / 2;
			}

			if (worldModel->getBall().getPos().getX() < -10)
			{
				maxDashPower = 40;		
				if (worldModel->getBody().getStamina() < 2700)
					maxDashPower = worldModel->getBody().getStaminaIncMax();
				if (worldModel->getBody().getStamina() < 2300)
					maxDashPower = worldModel->getBody().getStaminaIncMax() / 3;
			}
						
			if (hypot(worldModel->getBody().getPos().getX() - homePoint.x, 
					worldModel->getBody().getPos().getY() - homePoint.y) < 7)
				maxDashPower = worldModel->getBody().getStaminaIncMax() / 2;

			if (hypot(worldModel->getBody().getPos().getX() - homePoint.x, 
					worldModel->getBody().getPos().getY() - homePoint.y) < 3)
				maxDashPower = 0;
		}
		else
		{
			if (hypot(worldModel->getBody().getPos().getX() - homePoint.x, 
					worldModel->getBody().getPos().getY() - homePoint.y) < 5)
				maxDashPower = 80;

			if (hypot(worldModel->getBody().getPos().getX() - homePoint.x, 
					worldModel->getBody().getPos().getY() - homePoint.y) < 3)
				maxDashPower = 50;

			if (hypot(worldModel->getBody().getPos().getX() - homePoint.x, 
					worldModel->getBody().getPos().getY() - homePoint.y) < 1.5)
				maxDashPower = 0;
		}
	}
	
	float idealBodyDir = 0;
	
	if (worldModel->getPlayMode() != PM_PLAY_ON)
	{
		LOG << "Positioning NPO " << endl;
		if (worldModel->getPlayMode() == PM_OFFSIDE)
		{
			LOG << "Positioning  NPO Offside NPOX" << endl;
			ForbiddenAreaGotoPoint doGotoPoint(AT_POSITIONING, homePoint, worldModel->getBall().getPos(), 
				9.15, worldModel->getBody(), true);
			if (doGotoPoint.getStatus() == 0)
			{
				if (doGotoPoint.getCommand())
					delete doGotoPoint.getCommand();
				return TurnToPoint(AT_POSITIONING, worldModel->getBall().getPos(),
						worldModel->getBody(), 5).getCommand();
			}
			else
				return doGotoPoint.getCommand();
		}
		else if (worldModel->getPlayMode() == PM_GOAL_KICK_OPP)
		{
			LOG << "Positioning  NPO GoalKickOpp NPOX" << endl;
			ForbiddenAreaGotoPoint doGotoPoint(AT_POSITIONING, getOppGoalieKickHome(homePoint),
				   	36, -20, 20, worldModel->getBody());
			if (doGotoPoint.getStatus() == 0)
			{
				if (doGotoPoint.getCommand())
					delete doGotoPoint.getCommand();
				return TurnToPoint(AT_POSITIONING, worldModel->getBall().getPos(),
						worldModel->getBody(), 5).getCommand();
			}
			else
				return doGotoPoint.getCommand();
		}
		else if (worldModel->getPlayMode() == PM_FREE_KICK_OPP &&
				 worldModel->getVirtualPlayMode() == VPM_GOALIE_CATCH_BALL_OPP)
		{
			LOG << "Positioning  NPO GoalieCatchOpp NPOX" << endl;
			ForbiddenAreaGotoPoint doGotoPoint(AT_POSITIONING, getOppGoalieKickHome(homePoint),
					worldModel->getBall().getPos(), 9.15,
				   	worldModel->getBody());
			if (doGotoPoint.getStatus() == 0)
			{
				if (doGotoPoint.getCommand())
					delete doGotoPoint.getCommand();
				return TurnToPoint(AT_POSITIONING, worldModel->getBall().getPos(),
						worldModel->getBody(), 5).getCommand();
			}
			else
				return doGotoPoint.getCommand();
		}
		else
		{
			LOG << "Positioning  NPO Other  NPOX" << endl;
			ForbiddenAreaGotoPoint doGotoPoint(AT_POSITIONING, homePoint, worldModel->getBall().
					getPos(), 9.15, worldModel->getBody());
			if (doGotoPoint.getStatus() == 0)
			{
				if (doGotoPoint.getCommand())
					delete doGotoPoint.getCommand();
				return TurnToPoint(AT_POSITIONING, worldModel->getBall().getPos(),
						worldModel->getBody(), 5).getCommand();
			}
			else
				return doGotoPoint.getCommand();
		}

	}
	
	if (worldModel->getBody().getPostNum() >= 9)
	{
		idealBodyDir = 0; 
		return DashNormalGotoPoint(AT_POSITIONING, homePoint, 1.5, worldModel->getBody(), idealBodyDir,
				maxDashPower).getCommand();
	}

	if ((worldModel->getBody().getPostNum() <= 5 || 
		(worldModel->getBody().getPostNum() == 8 && 
		worldModel->getBall().getPos().getX() < 10)) &&
		worldModel->getBall().getPos().getX() > -32)
	{
		if (worldModel->getBall().getPos().getY() > homePoint.y)
			idealBodyDir = 90; 
		else
			idealBodyDir = -90;

		if (worldModel->getBody().getPostNum() == 8)
		{
			if (fabs(worldModel->getBall().getPos().getY()) > 12)
				idealBodyDir = 180;
		}
		if (worldModel->getBody().getPostNum() == 4)
		{
			if (worldModel->getBall().getPos().getY() < 0)
				idealBodyDir = 180;

			if (worldModel->getBall().getPos().getY() > 15)
				idealBodyDir = 135;
		}
		if (worldModel->getBody().getPostNum() == 3)
		{
			if (worldModel->getBall().getPos().getY() > 0)
				idealBodyDir = 180;

			if (worldModel->getBall().getPos().getY() < -15)
				idealBodyDir = -135;
		}
		if (worldModel->getBody().getPostNum() == 5)
		{
			if (worldModel->getBall().getPos().getY() < 12)
				idealBodyDir = 180;
		}
		if (worldModel->getBody().getPostNum() == 2)
		{
			if (worldModel->getBall().getPos().getY() > -12)
				idealBodyDir = 180;
		}
		
		return DashNormalGotoPoint(AT_POSITIONING, homePoint, 1.5, worldModel->getBody(), idealBodyDir,
				maxDashPower).getCommand();
	}

	return DashNormalGotoPoint(AT_POSITIONING, homePoint, 1.5, worldModel->getBody(),
			maxDashPower).getCommand();
	
}





float Positioning::getMaxXFor34()
{
	float maxXFor34 = 0;
	unsigned i;
	float xTmmFullPlayer[FULL_PLAYERS_NUM];
	float xTmmHalfPlayer[HALF_PLAYERS_NUM];

	for (int i = 0; i < FULL_PLAYERS_NUM; i++)
		xTmmFullPlayer[i] = 0;	
	
	for (int i = 0; i < HALF_PLAYERS_NUM; i++)
		xTmmHalfPlayer[i] = 0;	
	
	if (worldModel->getBody().getPostNum() == 3)
	{
		xTmmFullPlayer[7] = 0;
	
		if (worldModel->getFullPlayer(TID_TEAMMATE, 6).isValid()) 
			xTmmFullPlayer[6] = worldModel->getFullPlayer(TID_TEAMMATE, 6).getPos().
							getX() + 15;
		else
			xTmmFullPlayer[6] = 0;
	}
	
	if (worldModel->getBody().getPostNum() == 4)
	{
		if (worldModel->getFullPlayer(TID_TEAMMATE, 7).isValid()) 
			xTmmFullPlayer[7] = worldModel->getFullPlayer(TID_TEAMMATE, 7).getPos().
							getX() + 15;
		else
			xTmmFullPlayer[7] = 0;

		xTmmFullPlayer[6] = 0;
	}
		
	
	
	for (int i = 0; i < FULL_PLAYERS_NUM; i++)
		if (i != 6 && i != 7)	
			if (worldModel->getFullPlayer(TID_TEAMMATE,i).isValid() &&
				!worldModel->getFullPlayer(TID_TEAMMATE,i).isGoalie())
				xTmmFullPlayer[i] = worldModel->getFullPlayer(TID_TEAMMATE,i).
										getPos().getX();				

	xTmmFullPlayer[4] = 0;
	
	for (int i = 0; i < HALF_PLAYERS_NUM; i++)
		if (worldModel->getHalfPlayer(TID_TEAMMATE,i).isValid() &&
			!worldModel->getHalfPlayer(TID_TEAMMATE,i).isGoalie())
			xTmmHalfPlayer[i] = worldModel->getHalfPlayer(TID_TEAMMATE,i).
								getPos().getX();				


	bool isGoalieInList = false;
	
	for (int i = 0; i < FULL_PLAYERS_NUM; i++)
		if (worldModel->getFullPlayer(TID_TEAMMATE,i).isValid() &&
			worldModel->getFullPlayer(TID_TEAMMATE,i).isGoalie())
			isGoalieInList = true;

	for (int i = 0; i < HALF_PLAYERS_NUM; i++)
		if (worldModel->getHalfPlayer(TID_TEAMMATE,i).isValid() &&
			worldModel->getHalfPlayer(TID_TEAMMATE,i).isGoalie())
			isGoalieInList = true;
			
			
	
	if (isGoalieInList)
	{
		maxXFor34 = 0xFFFF;
		for (i = 0; i < FULL_PLAYERS_NUM; i++)
			if (xTmmFullPlayer[i] < maxXFor34)
				maxXFor34 = xTmmFullPlayer[i];

		for (i = 0; i < HALF_PLAYERS_NUM; i++)
			if (xTmmHalfPlayer[i] < maxXFor34)
				maxXFor34 = xTmmHalfPlayer[i];
	}
	else
	{
		float firstMinX = 0xFFFF, secMinX = 0xFFFF;
		for (i = 0; i < FULL_PLAYERS_NUM; i++)
			if (xTmmFullPlayer[i] < secMinX)
				{
					secMinX = xTmmFullPlayer[i];
					if (secMinX < firstMinX)
						swap(firstMinX, secMinX);
				}

		for (i = 0; i < HALF_PLAYERS_NUM; i++)
			if (xTmmHalfPlayer[i] < secMinX)
				{
					secMinX = xTmmHalfPlayer[i];
					if (secMinX < firstMinX)
						swap(firstMinX, secMinX);
				}
		maxXFor34 = secMinX;		
	
	}


	if (maxXFor34 > 0)
		maxXFor34 = 0;
		
	return maxXFor34;
}


float Positioning::getMaxXFor7()
{
	float maxXFor7 = 0;
	unsigned i;
	float xTmmFullPlayer[FULL_PLAYERS_NUM];
	float xTmmHalfPlayer[HALF_PLAYERS_NUM];

	for (int i = 0; i < FULL_PLAYERS_NUM; i++)
		xTmmFullPlayer[i] = 0;	
	
	for (int i = 0; i < HALF_PLAYERS_NUM; i++)
		xTmmHalfPlayer[i] = 0;	
	
	/////.
		xTmmFullPlayer[4] = 0;
	////.
	
		
	
	
	for (int i = 0; i < FULL_PLAYERS_NUM; i++)
		if (i != 4)	
			if (worldModel->getFullPlayer(TID_TEAMMATE,i).isValid() &&
				!worldModel->getFullPlayer(TID_TEAMMATE,i).isGoalie())
				xTmmFullPlayer[i] = worldModel->getFullPlayer(TID_TEAMMATE,i).
										getPos().getX();				

	for (int i = 0; i < HALF_PLAYERS_NUM; i++)
		if (worldModel->getHalfPlayer(TID_TEAMMATE,i).isValid() &&
			!worldModel->getHalfPlayer(TID_TEAMMATE,i).isGoalie())
			xTmmHalfPlayer[i] = worldModel->getHalfPlayer(TID_TEAMMATE,i).
								getPos().getX();				


	bool isGoalieInList = false;
	
	for (int i = 0; i < FULL_PLAYERS_NUM; i++)
		if (worldModel->getFullPlayer(TID_TEAMMATE,i).isValid() &&
			worldModel->getFullPlayer(TID_TEAMMATE,i).isGoalie())
			isGoalieInList = true;

	for (int i = 0; i < HALF_PLAYERS_NUM; i++)
		if (worldModel->getHalfPlayer(TID_TEAMMATE,i).isValid() &&
			worldModel->getHalfPlayer(TID_TEAMMATE,i).isGoalie())
			isGoalieInList = true;
			
			
	
	if (isGoalieInList)
	{
		maxXFor7 = 0xFFFF;
		for (i = 0; i < FULL_PLAYERS_NUM; i++)
			if (xTmmFullPlayer[i] < maxXFor7)
				maxXFor7 = xTmmFullPlayer[i];

		for (i = 0; i < HALF_PLAYERS_NUM; i++)
			if (xTmmHalfPlayer[i] < maxXFor7)
				maxXFor7 = xTmmHalfPlayer[i];
	}
	else
	{
		float firstMinX = 0xFFFF, secMinX = 0xFFFF;
		for (i = 0; i < FULL_PLAYERS_NUM; i++)
			if (xTmmFullPlayer[i] < secMinX)
				{
					secMinX = xTmmFullPlayer[i];
					if (secMinX < firstMinX)
						swap(firstMinX, secMinX);
				}

		for (i = 0; i < HALF_PLAYERS_NUM; i++)
			if (xTmmHalfPlayer[i] < secMinX)
				{
					secMinX = xTmmHalfPlayer[i];
					if (secMinX < firstMinX)
						swap(firstMinX, secMinX);
				}
		maxXFor7 = secMinX;		
	
	}


	if (maxXFor7 > 0)
		maxXFor7 = 0;
		
	return maxXFor7;
}


Point Positioning::getBasicPos(unsigned num) const
{
	return basicPos[num];
}

Point Positioning::getHomePoint(unsigned playerUniNum) const
{
	return basicPos[Player::uniNumToPostNum(playerUniNum) - 1];
}

Point Positioning::getOppGoalieKickHome(Point homePoint) const
{
	Point newHomePoint;
	newHomePoint = homePoint;
	if (worldModel->getPlayMode() == PM_FREE_KICK_OPP &&
				 worldModel->getVirtualPlayMode() == VPM_GOALIE_CATCH_BALL_OPP)
	{
		LOG << "Positioning GoalieCatchOppHome model=catch" << endl;
		if (worldModel->getBody().getPostNum() == 6)
			newHomePoint = Point(18, 10);
		if (worldModel->getBody().getPostNum() == 7)
			newHomePoint = Point(18, -10);
		if (worldModel->getBody().getPostNum() == 9)
			newHomePoint = Point(25, 0);
		if (worldModel->getBody().getPostNum() == 10)
			newHomePoint = Point(30, -20);
		if (worldModel->getBody().getPostNum() == 11)
			newHomePoint = Point(30, 20);
		newHomePoint.y += worldModel->getBall().getPos().getY() * .15;
	}
	if (worldModel->getPlayMode() == PM_GOAL_KICK_OPP)
	{
		LOG << "Positioning GoalieCatchOppHome model=goalkick" << endl;
		if (worldModel->getBody().getPostNum() == 6)
			newHomePoint = Point(28, 10);
		if (worldModel->getBody().getPostNum() == 7)
			newHomePoint = Point(28, -10);
		if (worldModel->getBody().getPostNum() == 9)
			newHomePoint = Point(35, 0);
		if (worldModel->getBody().getPostNum() == 10)
			newHomePoint = Point(35, -20);
		if (worldModel->getBody().getPostNum() == 11)
			newHomePoint = Point(35, 20);
		newHomePoint.y += worldModel->getBall().getPos().getY() * .15;
	}
	return newHomePoint;
}
