/*
 *  Copyright 2002-2005, Mersad Team, Allameh Helli High School (NODET).
 *
 *  This program is free software, you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Library General Public License for more details.
 *
 *  This file is created by: Ahmad Boorghany
 *
 *  Released on Monday 1 August 2005, 10 Mordad 1384 by Mersad RoboCup Team.
 *  For more information please read README file.
*/

#ifndef __BODY_DECISION_H
#define __BODY_DECISION_H


#include <vector>

#include <SayDecision.h>
#include <HeadDecision.h>
#include <AdvancedAction.h>


class Form;

struct AdvancedActionWeights
{
	float alpha;
	float power;
	float rate;

	AdvancedActionWeights(float alpha, float power, float rate):
			alpha(alpha), power(power), rate(rate) {}
};

class BodyDecision
{
protected:
	Command *command;

	std::vector<AdvancedAction *> advancedActions;
	std::vector<AdvancedActionWeights> weights;
	std::vector<int> quickLevels;
	std::vector<float> values;

	void addAdvancedAction(AdvancedAction *advancedAction,
			AdvancedActionWeights weight, int quickLevel);

	const WorldModel *worldModel;

	void resetValues();
	void resetWeights();
	void resetQuickLevels();

public:
	BodyDecision(const WorldModel *worldModel);
	virtual ~BodyDecision();

	void updateAfterSenseBody();
	virtual void decide(int quickLevel, Form &form,	const Library &library);

	Command *getCommand();
};

#endif // __BODY_DECISION_H

