/*
 *  Copyright 2002-2005, Mersad Team, Allameh Helli High School (NODET).
 *
 *  This program is free software, you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Library General Public License for more details.
 *
 *  This file is created by: Ahmad Boorghany
 *  and is modified by: Meisam Vosoughpour, Sassan Haradji
 *
 *  Released on Monday 1 August 2005, 10 Mordad 1384 by Mersad RoboCup Team.
 *  For more information please read README file.
*/

#include <cmath>

#include <Degree.h>
#include <Logger.h>
#include <Defines.h>
#include <GroupWorks.h>
#include <SyncedAgent.h>

#include <AdvancedAgent.h>
#include <Basics.h>
#include <cassert>

using namespace std;
using namespace Basics;

ostream &operator<<(ostream &stream, const WithoutBallSelection &var)
{
	switch (var)
	{
	case WOBS_LOOKFORBALL:
		stream << "WOBS_LOOKFORBALL";
		break;

	case WOBS_BLOCK:
		stream << "WOBS_BLOCK";
		break;

	case WOBS_INTERCEPT:
		stream << "WOBS_INTERCEPT";
		break;

	case WOBS_POSITIONING:
		stream << "WOBS_POSITIONING";
		break;

	case WOBS_OFFENSE:
		stream << "WOBS_OFFENSE";
		break;

	case WOBS_DEFENSE:
		stream << "WOBS_DEFENSE";
		break;

	default:
		assert(0);
	}

	return stream;
}


// class GroupWorks
GroupWorks::GroupWorks(const WorldModel *worldModel): worldModel(worldModel)
{
	interceptNum = 0;
	interceptBlock = false;
	lastPassHearTime = 0;

	lastSRPDribbleTime = 0;

//	virtualKickCalc.init();
//	virtualPass.setCurKickCalculate(&virtualKickCalc);
}

void GroupWorks::updateInterceptNum(Library &library)
{
	if (library.lastBodyCycleCommand &&
		library.lastBodyCycleCommand->getCreator() == AT_SRPDRIBBLE)
		lastSRPDribbleTime = worldModel->getCurTime() - 1;

	if (worldModel->getBallStatus() == BS_KICKABLE_BODY ||
		worldModel->getBallStatus() == BS_KICKABLE_BODY_TMM ||
		worldModel->getBallStatus() == BS_KICKABLE_BODY_OPP)
	{
		LOG << "I have the ball now !" << endl;

		interceptBlock = false;
		interceptSay = true;
		interceptNum = worldModel->getBody().getUniNum();

		LOG << "final interceptNum (Bl, Sy, Pn):"
			<< " " << interceptBlock
			<< " " << interceptSay
			<< " " << interceptNum << endl;
		return;
	}

	if (!getChangeInterceptNumPermission())
	{
		LOG << "Changing interceptNum is not permitted." << endl;

		LOG << "final interceptNum (Bl, Sy, Pn):"
			<< " " << interceptBlock
			<< " " << interceptSay
			<< " " << interceptNum << endl;
		return;
	}

	bool halfPlayerFlag;
	float maxBlockWeight = 0;
	float curBlockWeight = 0;
	unsigned blocker;
	unsigned checkedTmmNum;
	unsigned checkedOppNum;
	unsigned defenserPlayer;
	Point ballStopPoint;
	const Player *checkPlayer;

	LOG << "GroupWorks::updateInterceptNum" << endl;

	checkedTmmNum = worldModel->getCurInterCalculate().getTmmCheckedPlayersNum();
	checkedOppNum = worldModel->getCurInterCalculate().getOppCheckedPlayersNum();

	if (checkedTmmNum > 0 || checkedOppNum > 0)
		ballStopPoint = worldModel->getCurInterCalculate().getFastestInterPlayer().getInterPoint();


	// cleaning interceptNum

	// intercept conditions
	// condition 1
	if (interceptNum && !interceptBlock)
	{
		if (interceptNum == worldModel->getBody().getUniNum())
		{
			if (worldModel->getCurInterCalculate().getCheckedListNum(
				worldModel->getBody()) == NOVALUE)
			{
				interceptNum = 0;
				LOG << "interceptNum reset 1.1" << endl;
			}
		}
		else if (worldModel->getFullPlayer(TID_TEAMMATE, interceptNum - 1).getPosDeviation() <= 3 &&
				 worldModel->getCurInterCalculate().getCheckedListNum(
				 		worldModel->getFullPlayer(TID_TEAMMATE, interceptNum - 1)) == NOVALUE)
		{
			interceptNum = 0;
			LOG << "interceptNum reset 1.2" << endl;
		}
	}

	// block and intercept conditions
	// condition 2
	if (worldModel->getBall().getPosDeviation() > 5)
	{
		interceptNum = 0;
		LOG << "interceptNum reset 2" << endl;
	}

	// block conditions
	// condition 3
	if (interceptNum && interceptBlock && checkedTmmNum > 0)
	{
		if (interceptNum == worldModel->getBody().getUniNum() &&
			worldModel->getCurInterCalculate().getCheckedListNum(
					worldModel->getBody()) == NOVALUE)
		{
			interceptNum = 0;
			LOG << "interceptNum reset 3.1" << endl;
		}

		else if (worldModel->getFullPlayer(TID_TEAMMATE, interceptNum - 1).getPosDeviation() <= 3 &&
					worldModel->getCurInterCalculate().getCheckedListNum(
					worldModel->getFullPlayer(TID_TEAMMATE, interceptNum - 1)) == NOVALUE)
		{
			interceptNum = 0;
			LOG << "interceptNum reset 3.2" << endl;
		}
	}

	// Condition 4
	if (interceptNum && interceptBlock && 
		(checkedTmmNum > 0 || checkedOppNum > 0))
	{
		if (interceptNum == worldModel->getBody().getUniNum())
			checkPlayer = &worldModel->getBody();
		else
			checkPlayer = &worldModel->getFullPlayer(TID_TEAMMATE, interceptNum - 1);

		// Checking Teammates
		for (unsigned i = 0; i < FULL_PLAYERS_NUM; i++)
		{
			if (!worldModel->getFullPlayer(TID_TEAMMATE, i).isValid())
				continue;

			if (worldModel->getFullPlayer(TID_TEAMMATE, i).isGoalie())
				continue;

			if (abs(getBlockWeight(ballStopPoint, worldModel->getFullPlayer(TID_TEAMMATE, i)) -
					 getBlockWeight(ballStopPoint, *checkPlayer)) > 10 &&
				getBlockWeight(ballStopPoint, worldModel->getFullPlayer(TID_TEAMMATE, i)) > 
				getBlockWeight(ballStopPoint, *checkPlayer))
			{
				interceptNum = 0;
				LOG << "interceptNum reset 4.1" << endl;

				break;
			}
		}

		// Checking Body
		if (interceptNum && !worldModel->getBody().isGoalie() &&
			abs(getBlockWeight(ballStopPoint, worldModel->getBody()) -
				 getBlockWeight(ballStopPoint, *checkPlayer)) > 10 &&
			getBlockWeight(ballStopPoint, worldModel->getBody()) >
			getBlockWeight(ballStopPoint, *checkPlayer))
		{
			interceptNum = 0;
			LOG << "interceptNum reset 4.2" << endl;
		}
	}


	// making interceptNum
	if (worldModel->getBall().getPosDeviation() > 10)
	{
		LOG << "I have not enough information to make a choice." << endl;

		return;
	}

	// intercept conditions
	// condition 1
	if (interceptNum == 0 && checkedTmmNum == 1 &&
		worldModel->getCurInterCalculate().getFastestPlayer().getTeamId() == TID_TEAMMATE &&
		worldModel->getCurInterCalculate().getFastestPlayer().getModel() == PLM_FULL)
	{ // somebody is the one Fastest.
		interceptNum = worldModel->getCurInterCalculate().getFastestPlayer().getUniNum();
		interceptBlock = false;
		interceptSay = true;

		LOG << "interceptNum set 1" << endl;
	}

	// condition 2
	if (interceptNum == 0 && checkedTmmNum > 1 &&
		worldModel->getCurInterCalculate().getFastestPlayer().getTeamId() == TID_TEAMMATE)
	{ // A few Tmms are Fastest.
		defenserPlayer = NOVALUE;
		halfPlayerFlag = false;

		for (unsigned i = 0; i < worldModel->getCurInterCalculate().getCheckedPlayersNum(); i++)
		{
			if (worldModel->getCurInterCalculate().getCheckedInterPlayer(i).getPlayer().getTeamId() ==
					TID_OPPONENT)
				continue;

			if (checkedOppNum != 0 &&
				worldModel->getCurInterCalculate().getCheckedInterPlayer(i).getInterTime() >
					worldModel->getCurInterCalculate().getFastestOppInterPlayer().getInterTime())
				continue;
			
			if (worldModel->getCurInterCalculate().getCheckedInterPlayer(i).getPlayer().getModel() ==
					PLM_HALF)
				halfPlayerFlag = true;
			else
			{
				if (defenserPlayer == NOVALUE)
					defenserPlayer = i;
				else if (worldModel->getCurInterCalculate().getCheckedInterPlayer(i).getPlayer().getPostNum() <
						 	worldModel->getCurInterCalculate().getCheckedInterPlayer(defenserPlayer).getPlayer().getPostNum())
					defenserPlayer = i;
			}
		}

		// setting results
		if (defenserPlayer != NOVALUE)
		{
			if (halfPlayerFlag == false)
			{
				interceptNum = worldModel->getCurInterCalculate().getCheckedInterPlayer(defenserPlayer).getPlayer().getUniNum();
				interceptBlock = false;
				interceptSay = true;

				LOG << "interceptNum set 2.1" << endl;
			}
			else if (worldModel->getCurInterCalculate().getCheckedInterPlayer(
					defenserPlayer).getPlayer().getUniNum() ==
					worldModel->getBody().getUniNum()) // I think I must go to ball but I am not sure.
			{
				interceptNum = worldModel->getBody().getUniNum();
				interceptBlock = false;
				interceptSay = false;

				LOG << "interceptNum set 2.2" << endl;
			}
		}
	}

	// block Conditions
	// condition 3
	if (interceptNum == 0 &&
		(worldModel->getBallStatus() == BS_KICKABLE_OPP ||
		 worldModel->getBallStatus() == BS_FREE_BALL_OPP))
	{
		blocker = NOVALUE;
		halfPlayerFlag = false;

		// Checking Teammates
		for (unsigned i = 0; i < FULL_PLAYERS_NUM; i++)
		{
			if (!worldModel->getFullPlayer(TID_TEAMMATE, i).isValid())
				continue;

			if (worldModel->getFullPlayer(TID_TEAMMATE, i).isGoalie())
				continue;

			curBlockWeight = getBlockWeight(ballStopPoint,
					worldModel->getFullPlayer(TID_TEAMMATE, i));

			if (blocker == NOVALUE || curBlockWeight > maxBlockWeight)
			{
				blocker = i;
				maxBlockWeight = curBlockWeight;
			}

		}

		// Checking Body
		if (!worldModel->getBody().isGoalie())
		{
			curBlockWeight = getBlockWeight(ballStopPoint,
					worldModel->getBody());

			if (blocker == NOVALUE || curBlockWeight > maxBlockWeight)
			{
				blocker = worldModel->getBody().getUniNum() - 1;
				maxBlockWeight = curBlockWeight;
			}
		}

		// Checking Half Players
		if (blocker != NOVALUE)
			for (unsigned i = 0; i < HALF_PLAYERS_NUM; i++)
			{
				if (!worldModel->getHalfPlayer(TID_TEAMMATE, i).isValid())
					continue;

				if (getBlockWeight(ballStopPoint,
						worldModel->getHalfPlayer(TID_TEAMMATE, i)) >
						maxBlockWeight)
				{
					halfPlayerFlag = true;
					break;
				}
			}

		// Setting results
		if (blocker == NOVALUE)
		{
			interceptNum = 0;
			interceptBlock = true;
			interceptSay = false;
		}
		else if (halfPlayerFlag == false)
		{
			interceptNum = blocker + 1;
			interceptBlock = true;
			interceptSay = true;

			LOG << "interceptNum set 3.1" << endl;
		}
		else if (blocker + 1 == worldModel->getBody().getUniNum()) // I think I must block but I am not sure.
		{
			interceptNum = worldModel->getBody().getUniNum();
			interceptBlock = true;
			interceptSay = false;

			LOG << "interceptNum set 3.2" << endl;
		}
	}

	LOG << "final interceptNum (Bl, Sy, Pn):"
		<< " " << interceptBlock
		<< " " << interceptSay
		<< " " << interceptNum << endl;

}

float GroupWorks::getBlockWeight(const Point &ballStopPoint,
		const Player &player) const
{
	Vector playerVec;
	Vector goalVec;
	float dirFactor;
	float distFactor;
	
	playerVec.setByPoints(player.getPos(), ballStopPoint);
	goalVec.setByPoints(Point(-52.5, 0), ballStopPoint);

	distFactor = playerVec.getMagnitude();
	dirFactor = abs(Degree::getDeltaAngle(goalVec.getDirection(),
			playerVec.getDirection()));

	return 100 - 0.15 * dirFactor - 1 * distFactor;
}

void GroupWorks::setInterceptNumByPassSay(unsigned uniNum)
{
	interceptSay = true;
	lastPassHearTime = worldModel->getCurTime();

	if (uniNum <= 11) // GoToBall Mode
	{
		interceptBlock = false;
		interceptNum = uniNum;
	}
	else // Block Mode
	{
		interceptBlock = true;
		interceptNum = uniNum - 11;
	}

	LOG << "setting interceptNum (Bl, Sy, Pn):"
		<< " " << interceptBlock
		<< " " << interceptSay
		<< " " << interceptNum << endl;
}

void GroupWorks::resetByChangePlayMode()
{
	interceptNum = 0;
	interceptBlock = false;
	interceptSay = false;
	lastPassHearTime = 0;
}

void GroupWorks::changeInterceptNumByHear(unsigned sender, unsigned uniNum)
{
	if (!getChangeInterceptNumPermission())
	{
		LOG << "Changing interceptNum is not permitted." << endl;
		return;
	}

	bool setFlag = false;

	LOG << "suggested interceptNum: " << uniNum << endl;

	if (sender == interceptNum ||
		sender == worldModel->getBody().getUniNum())
		setFlag = true;
	else if (!uniNum)
		return; // No suggestion.

	else if (uniNum == worldModel->getBody().getUniNum())
		setFlag = true;

	else if (worldModel->getFullPlayer(TID_TEAMMATE, uniNum - 1).getPosDeviation() > 2 ||
			 worldModel->getFullPlayer(TID_TEAMMATE, interceptNum - 1).getPosDeviation() > 2)
		setFlag = true;

	if (setFlag)
	{
		if (uniNum <= 11) // GoToBall Mode
		{
			interceptNum = uniNum;
			interceptBlock = false;
			interceptSay = true;
		}
		else // Block Mode
		{
			interceptNum = uniNum - 11;
			interceptBlock = true;
			interceptSay = true;
		}

		LOG << "setting interceptNum (Bl, Sy, Pn):"
			<< " " << interceptBlock
			<< " " << interceptSay
			<< " " << interceptNum << endl;
	}
}

WithoutBallSelection GroupWorks::getWithoutBallSelection(
		const Library &library) const
{
	WithoutBallSelection selection;

	if (!worldModel->getBall().isValid())
		selection = WOBS_LOOKFORBALL;
	else if (interceptNum == worldModel->getBody().getUniNum())
	{
		if (interceptBlock)
			selection = WOBS_BLOCK;
		else
			selection = WOBS_INTERCEPT;
	}
	else
		selection = WOBS_POSITIONING;

	if (!getChangeInterceptNumPermission())
	{
		SpecifyOffenseDefense(selection);

		LOG << "GroupWorks::getWithoutBallSelection > return: "
			<< selection << endl;

		return selection;
	}

	if (lastSRPDribbleTime > worldModel->getCurTime() - 5)
		selection = WOBS_INTERCEPT;

	LOG << " Meisam2005 GroupWorks1 : selection = " << selection << endl;

	if (selection == WOBS_POSITIONING && 
		(worldModel->getBallStatus() == BS_FREE_BALL_BODY /*||
		worldModel->getBallStatus() == BS_FREE_BALL_BODY_TMM*/))
		selection = WOBS_INTERCEPT; // Be careful of this line.
 
	if (selection == WOBS_INTERCEPT)	
	{
		if (worldModel->getBallStatus() == BS_KICKABLE_OPP || 
			worldModel->getBallStatus() == BS_FREE_BALL_OPP)
			selection = WOBS_BLOCK;
	
		if (worldModel->getBody().getPostNum() <= 5)
			if (worldModel->getBallStatus() == BS_FREE_BALL_BODY_OPP ||
				 worldModel->getBallStatus() == BS_FREE_BALL_TMM_OPP)
				selection = WOBS_BLOCK;
	}
//	LOG << "SALAM man asabaniam : " << (bool)(worldModel->getBallStatus() 
//		== BS_FREE_BALL_TMM_OPP) << endl;
	if (selection == WOBS_POSITIONING &&
		(worldModel->getBody().getPostNum() <= 5 ||
			(worldModel->getBody().getPostNum() == 8 && 
			worldModel->getBall().getPos().getX() < 5 &&
			worldModel->getBallStatus() != BS_FREE_BALL_BODY_OPP)))
	{
		if (worldModel->getBallStatus() == BS_KICKABLE_OPP || 
			worldModel->getBallStatus() == BS_FREE_BALL_OPP ||
			worldModel->getBallStatus() == BS_FREE_BALL_BODY_OPP ||
			worldModel->getBallStatus() == BS_FREE_BALL_TMM_OPP)
		{
			float homeDeltaX = worldModel->getBall().getPos().getX() - 
							library.positioning.getHomePoint(
							worldModel->getBody().getUniNum()).x;
					
			LOG << "Meisam2005 : aaaaah : homePoint : " << 
							library.positioning.getHomePoint(
							worldModel->getBody().getUniNum()) << endl;
				
						
			float homeDeltaY = worldModel->getBall().getPos().getY() - 
							library.positioning.getHomePoint(
							worldModel->getBody().getUniNum()).y;
			
			LOG << "Meisam2005 Intelligent Section of GrourWorks :-----------------------------------" << endl;
			LOG << "DeltaX : " << homeDeltaX << endl;
			LOG << "DeltaY : " << homeDeltaY << endl;
			LOG << "----------------------------------------------------------------------" << endl;
			
			if (worldModel->getBody().getPostNum() == 2)
				if (homeDeltaY < 6 && (homeDeltaX < 8 || 
						worldModel->getBody().getPos().getX() > 
						worldModel->getBall().getPos().getX()))
				{
					LOG << "Meisam2005 intelligent su..1 " << endl;		
					selection = WOBS_BLOCK;
				}

			if (worldModel->getBody().getPostNum() == 3)
			{
		    if (worldModel->getBall().getPos().getX() > 0)
		    {
					if (fabs(homeDeltaY) < 6 && (homeDeltaX < 6 || 
							worldModel->getBody().getPos().getX() > 
							worldModel->getBall().getPos().getX()))
					{
					LOG << "Meisam2005 intelligent su..2 " << endl;		
						selection = WOBS_BLOCK;
					}

		    }
		    else	
					if (abs(homeDeltaY) < 10 && (homeDeltaX < 10 || 
							worldModel->getBody().getPos().getX() > 
							worldModel->getBall().getPos().getX()))
					{
					LOG << "Meisam2005 intelligent su..3 " << endl;		
						selection = WOBS_BLOCK;
					}
			}
			if (worldModel->getBody().getPostNum() == 4)
			{
		    	if (worldModel->getBall().getPos().getX() > 0)
		    	{
					if (fabs(homeDeltaY) < 6 && (homeDeltaX < 6 || 
							worldModel->getBody().getPos().getX() > 
							worldModel->getBall().getPos().getX()))
					{
						LOG << "Meisam2005 intelligent su..4 " << endl;		
						selection = WOBS_BLOCK;
					}

		    	}
		    	else	
					if (abs(homeDeltaY) < 10 && (homeDeltaX < 10 || 
							worldModel->getBody().getPos().getX() > 
							worldModel->getBall().getPos().getX()))
					{
						LOG << "Meisam2005 intelligent su..5 " << endl;		
						selection = WOBS_BLOCK;
					}
			}
			
			if (worldModel->getBody().getPostNum() == 5)
				if (homeDeltaY > -6 && (homeDeltaX < 8 || 
						worldModel->getBody().getPos().getX() > 
						worldModel->getBall().getPos().getX()))
				{
					LOG << "Meisam2005 intelligent su..6 " << endl;		
					selection = WOBS_BLOCK;
				}

			if (worldModel->getBody().getPostNum() == 8)
				if (abs(homeDeltaY) < 12 && (homeDeltaX < 6 || 
						worldModel->getBody().getPos().getX() > 
						worldModel->getBall().getPos().getX()))
				{
					LOG << "Meisam2005 intelligent su..7 " << endl;		
					selection = WOBS_BLOCK;
				}
		}
	}





//////////////////////////BLOCK FOR 6 & 7 : begin
	if (selection == WOBS_POSITIONING &&
		(worldModel->getBody().getPostNum() == 6 ||
			worldModel->getBody().getPostNum() == 7))
	{
		if (worldModel->getBallStatus() == BS_KICKABLE_OPP || 
			worldModel->getBallStatus() == BS_FREE_BALL_OPP ||
			worldModel->getBallStatus() == BS_FREE_BALL_BODY_OPP ||
			worldModel->getBallStatus() == BS_FREE_BALL_TMM_OPP)
		{
			Point localHomePoint;
			localHomePoint = library.positioning.getHomePoint(
							worldModel->getBody().getUniNum());

			float homeDeltaX = worldModel->getBall().getPos().getX() - 
							localHomePoint.x;
						
			float homeDeltaY = worldModel->getBall().getPos().getY() - 
							localHomePoint.y;
			
/*			LOG << "Intelligent Section of GrourWorks :-----------------------------------" << endl;
			LOG << "DeltaX : " << homeDeltaX << endl;
			LOG << "DeltaY : " << homeDeltaY << endl;
			LOG << "----------------------------------------------------------------------" << endl;
*/
			if (worldModel->getBody().getPostNum() == 6 && 
				worldModel->getBall().getPos().getY() > 4.5 && 
				homeDeltaX > -10 && 
				(localHomePoint.x - worldModel->getBody().getPos().getX()) > -14			
				&& homeDeltaY > -10)
			{
					LOG << "Meisam2005 intelligent su..7 " << endl;		
				selection = WOBS_BLOCK;
			}

			if (worldModel->getBody().getPostNum() == 7 && 
				worldModel->getBall().getPos().getY() < -4.5 && 
				homeDeltaX > -10 && 
				(localHomePoint.x - worldModel->getBody().getPos().getX()) > -14
				&& homeDeltaY > -10)			
				
			{
					LOG << "Meisam2005 intelligent su..8 " << endl;		
				selection = WOBS_BLOCK;
			}
		}
	}
//////////////////////////BLOCK FOR 6 & 7 : end

	SpecifyOffenseDefense(selection);

	LOG << "GroupWorks::getWithoutBallSelection > return: "
		<< selection << endl;

	return selection;
}

void GroupWorks::signInterceptNum(SayDecisionForm &sayForm)
{
	if (interceptNum == 0)
	{
		sayForm.gwInterceptNum = 0;
		sayForm.gwAttentionNum = 0;
		return;
	}
	else
		sayForm.gwAttentionNum = interceptNum;

	if ((interceptNum == worldModel->getBody().getUniNum() ||
		 worldModel->getFullPlayer(TID_TEAMMATE, interceptNum - 1).
		 		getPosDeviation() < 2) &&
		worldModel->getBall().getPosDeviation() < 2 && interceptSay)
	{
		if (interceptBlock)
			sayForm.gwInterceptNum = interceptNum + 11; // It means Block Mode
		else
			sayForm.gwInterceptNum = interceptNum;
	}
	else
		sayForm.gwInterceptNum = 0; // I reset it because I am not sure.
}

bool GroupWorks::getChangeInterceptNumPermission() const
{
	if (worldModel->getBall().getSeeTime() > worldModel->getCurTime() - 2 &&
		worldModel->getCurTime() - lastPassHearTime > 2 &&
		(worldModel->getBallStatus() == BS_KICKABLE_TMM ||
		 worldModel->getBallStatus() == BS_KICKABLE_OPP ||
		 worldModel->getBallStatus() == BS_KICKABLE_BODY ||
		 worldModel->getBallStatus() == BS_KICKABLE_BODY_TMM ||
		 worldModel->getBallStatus() == BS_KICKABLE_BODY_OPP ||
		 worldModel->getBallStatus() == BS_KICKABLE_TMM_OPP))
		 return true;

	if (worldModel->getCurTime() - lastPassHearTime > 4 ||
		interceptNum == 0)
		return true;
	return false;
}

/*void GroupWorks::suggestPassSayManager()
{
	LOG << "GroupWorks::suggestPassSayManager" << endl;

	turnNeckMode = TNM_LOOK_NORMALLY_TO_BALL;
	sayFlags.attentionNum = hearedRequestPassSender;

	if (worldModel->getTimer().now() > ZERO_QUICK_LEVEL_MS ||
		worldModel->getCurTime() - worldModel->getBall().getSeeTime() > 2)
	{
		LOG << "Not enough time or informations." << endl;
		return; // my informations or my time are not enough.
	}

	const Player &kickPlayer = worldModel->getNearestTmmToBall();

	virtualKickCalc.update(worldModel, kickPlayer);
	float value = virtualPass.getValue(kickPlayer);

	if (value <= -50 || value == AD_MIN_VALUE)
	{
		LOG << "There is no good pass." << endl;
		return;
	}

	sayFlags.suggestPassSay = true;
	sayFlags.passDir = virtualPass.getPassInterCalc()->
			getFirstBall().getVel().getDirection();
	sayFlags.passSpeed = virtualPass.getPassInterCalc()->
			getFirstBall().getVel().getMagnitude();
	sayFlags.passWeight = value;
	sayFlags.passPlayerNum = virtualPass.getPassPlayer()->getUniNum();

	LOG << "VirtualPass:" << endl
		<< "\tPlayer: " << virtualPass.getPassPlayer()->getTeamId()
		<< " " << virtualPass.getPassPlayer()->getUniNum() << endl
		<< "\tWeight: " << virtualPass.getPassValue() << endl
		<< "\tModel: " << virtualPass.getPassModel() << endl
		<< "\tDir: " << sayFlags.passDir << endl
		<< "\tSpeed: " << sayFlags.passSpeed << endl;

}*/

/*bool GroupWorks::getPassSayRequestPermittion()
{
	unsigned bodyNum = worldModel->getCurInterCalculate().
			getCheckedListNum(worldModel->getBody());

	if (bodyNum == NOVALUE || worldModel->getCurInterCalculate().
			getCheckedInterPlayer(bodyNum).getInterTime() > 3)
		return false;

	return true;
}*/

/*void GroupWorks::requestSuggestPassSayManager()
{
	LOG << "GroupWorks::requestSuggestPassSayManager" << endl;
*/
/*	if (worldModel->getBody().getPostNum() == 6 ||
		worldModel->getBody().getPostNum() == 7)
		requestNum = Player::postNumToUniNum(8);
	else*//* if (worldModel->getBody().getPostNum() == 9 ||
		worldModel->getBody().getPostNum() == 10 ||
		worldModel->getBody().getPostNum() == 11)
	{
		if (abs(worldModel->getBall().getPos().getY()) > 5)
		{
			if (worldModel->getBall().getPos().getY() > 0)
				requestNum = Player::postNumToUniNum(6);
			else
				requestNum = Player::postNumToUniNum(7);
		}
		else
			requestNum = lastRequestNum;
	}

	sayFlags.attentionNum = requestNum;
	sayFlags.requestPassSayNum = requestNum;
}*/

void GroupWorks::updateAfterSenseBody()
{
//	lastRequestNum = requestNum;
}

void GroupWorks::decide()
{
	LOG << "GroupWorks::decide" << endl;

/*	if (getPassSayRequestPermittion())
		requestSuggestPassSayManager();

	if (hearedRequestPassTime == worldModel->getCurTime() &&
		hearedRequestPassNum == worldModel->getBody().getUniNum())
		suggestPassSayManager();*/
}

void GroupWorks::SpecifyOffenseDefense(WithoutBallSelection &selection) const
{
	if (selection == WOBS_POSITIONING)
	{
		if (interceptBlock)
			selection = WOBS_DEFENSE;
		else
			selection = WOBS_OFFENSE;
	}

	if (selection == WOBS_DEFENSE &&
			(isInPlayers(worldModel->getBody().getUniNum(), "234") ||
			(isInPlayers(worldModel->getBody().getUniNum(), "AB5") &&
			worldModel->getBody().getPos().getX() >= 20)))
		selection = WOBS_OFFENSE;

	if (selection == WOBS_OFFENSE &&
			(isInPlayers(worldModel->getBody().getUniNum(), "6789") ||
			(isInPlayers(worldModel->getBody().getUniNum(), "AB5") &&
			worldModel->getBody().getPos().getX() < -20)))
		 selection = WOBS_DEFENSE;
}

// setting functions
/*void GroupWorks::setHearedRequestPassNum(unsigned hearedRequestPassNumArg)
{
	hearedRequestPassNum = hearedRequestPassNumArg;
}

void GroupWorks::setHearedRequestPassTime(unsigned hearedRequestPassTimeArg)
{
	hearedRequestPassTime = hearedRequestPassTimeArg;
}

void GroupWorks::setHearedRequestPassSender(unsigned hearedRequestPassSenderArg)
{
	hearedRequestPassSender = hearedRequestPassSenderArg;
}

*/
