/*
 *  Copyright 2002-2005, Mersad Team, Allameh Helli High School (NODET).
 *
 *  This program is free software, you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Library General Public License for more details.
 *
 *  This file is created by: Ahmad Boorghany
 *
 *  Released on Monday 1 August 2005, 10 Mordad 1384 by Mersad RoboCup Team.
 *  For more information please read README file.
*/

#ifndef __SAY_DECISION_H
#define __SAY_DECISION_H


#include <WorldModel.h>


#define RELATED_PLAYER_X_RANGE 47.5
#define RELATED_PLAYER_Y_RANGE 30


struct SayDecisionForm
{
	bool doRoutinSay;
	bool beforePassSay;

	bool suggestPassSay;
	float passDir;
	float passSpeed;
	float passWeight;
	unsigned passPlayerNum;

	unsigned interceptNum;
	unsigned attentionNum;

	bool freeKickSay;
	int freeKickAdviceNumber;

	unsigned planStatus;
	unsigned gwInterceptNum;
	unsigned gwAttentionNum;

	bool planSay;
	std::string planHeader;
	std::string planInfo;

	bool radarSay;
	int radarPlayersNum;
	int ids[3];
	Vector radarPlayers[3];

	SayDecisionForm();
	void reset();
};

class Command;

class SayDecision
{
	const WorldModel *worldModel;
	Command *sayCommand;
	Command *attentionCommand;

public:
	SayDecision(const WorldModel *worldModel);
	~SayDecision();

	bool getSayRoutinBallPermission(const SayDecisionForm &sayForm);

	void saySuggestPass(const SayDecisionForm &sayForm);
	void sayBeforePass(const Command *bodyCycleCommand, const SayDecisionForm &sayForm);
	void sayBeforePassRelated(const Command *bodyCycleCommand, const SayDecisionForm &sayForm);
	void sayRoutinBall(const Command *bodyCycleCommand, const SayDecisionForm &sayForm);
	void sayFreeKick(const SayDecisionForm &sayForm);
	void sayPlan(const SayDecisionForm &sayForm);
	void sayRadar(const SayDecisionForm &sayForm);
	void decide(const Command *bodyCycleCommand,
			SayDecisionForm &sayForm);

	Command *getSayCommand();
	Command *getAttentionCommand();
};

#endif // __SAY_DECISION_H

