/*
 *  Copyright 2002-2005, Mersad Team, Allameh Helli High School (NODET).
 *
 *  This program is free software, you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Library General Public License for more details.
 *
 *  This file is created by: Meisam Vosoughpour, Mohammad Salehe
 *
 *  Released on Monday 1 August 2005, 10 Mordad 1384 by Mersad RoboCup Team.
 *  For more information please read README file.
*/

#include <cmath>

#include <Degree.h>
#include <Basics.h>
#include <Defines.h>
#include <BasicTurn.h>
#include <Line.h>
#include <Logger.h>
#include <BasicDash.h>


using namespace std;
using namespace Degree;

//#define FREEKICK_GOTO_POINT_LOG
#define FORBIDDENAREA_GOTO_POINT_LOG

DashDribbleGotoBall::DashDribbleGotoBall(ActionType creator, const Ball &ball, const Body &body): BasicAction(creator)
{
	Body simBody(body);
	Ball simBall(ball);
//	simBody = *body;
//	simBall = *ball;
	float kickableArea, virtualKickableArea;
	float dashPower;
	int dribbleGTBTime = 0;
//	Command dashCommand;
	
	kickableArea = body.getSize() + body.getKickableMargin() + ball.getSize();
	virtualKickableArea = kickableArea * 1.1; // rate 1.1 for caution 
	
	dashPower = 100;
	if (body.getStamina() < max(body.getRecoverDecThr(),
								 body.getEffortDecThr()) *
							 body.getStaminaMax() + 150) // 150 for caution
		dashPower = body.getStaminaIncMax();
	
	//dashCommand.dash(dashPower);	 
	Command *dashCommand = new DashCommand(creator, dashPower);
	
	while (simBall.getBodyVec().getMagnitude() >= virtualKickableArea &&
		   dribbleGTBTime < 30)
	{
		simBody.simulateByAction(dashCommand);
		simBody.simulateByDynamics();
		simBall.simulateByDynamics(simBody);	
		dribbleGTBTime++;
	}
	
	if (simBall.getBodyVec().getMagnitude() < virtualKickableArea)
		command = dashCommand;
	else
		command = NULL;
	// dar inja bayad gotoball konad : oon adame ke mikhad in ro seda kone bayad aghel bashe. ;).
}

Command *DashDribbleGotoBall::getCommand()
{
	return command;
}

void ExactGotoPoint::setDashPower(const Body &body, Point targetPoint, 
								float defaultDashPower, float &dashPower)
{
	Body stoppedBody;
	float localDistance;
	
	stoppedBody = body;
	while (stoppedBody.getVel().getMagnitude() > 0.01)
		stoppedBody.simulateByDynamics();
	if (defaultDashPower >= 0)
	{
		localDistance = stoppedBody.getDistance(targetPoint);

#ifdef FREEKICK_GOTO_POINT_LOG
		LOG << "ExactGTP localDistance = " << localDistance << endl;
#endif

		dashPower = ((localDistance * (1 - body.getDecay())) / body.getDashPowerRate())
				/ body.getEffort();

#ifdef FREEKICK_GOTO_POINT_LOG
		LOG << "ExactGTP dashPower = " << dashPower << endl;
#endif

		if (dashPower > body.getMaxPower())
			dashPower = body.getMaxPower();

		if (dashPower > body.getStaminaIncMax() && body.getStamina() < 1350)
	   		dashPower = body.getStaminaIncMax();
	}
	else
	{
		localDistance = stoppedBody.getDistance(targetPoint);
		dashPower = (-localDistance * (1 - body.getDecay()) / body.getDashPowerRate())
				/ body.getEffort();;
		if (dashPower < body.getMinPower())
			dashPower = body.getMinPower();
		if (fabs(dashPower) > (body.getStaminaIncMax() / 2) && 
										body.getStamina() < 1400)
			dashPower = -(body.getStaminaIncMax() / 2);
	}
	
}

ExactGotoPoint::ExactGotoPoint(ActionType creator, Point targetPoint, float deltaDistance, 
		const Body &body, bool negDashFlag): BasicAction(creator)
{
	float pointDistance = hypot(targetPoint.x - body.getPos().getX(), 
								targetPoint.y - body.getPos().getY());
								
	float deltaAngleFront, deltaAngleBack, angleFront, angleBack;
	float cautionAngleBack, cautionAngleFront;
	float dashPower;
	
	command = NULL;
	mode = CT_EMPTY;

	cautionAngleBack = 20;
	cautionAngleFront = 20;
	
	angleFront = body.getBodyDir();
	angleBack = body.getBodyDir() - 180;
	angleBack = Degree::normalizeAngle(angleBack);
	
	if (pointDistance >= deltaDistance)
	{
		Vector pointVector;
		pointVector.setByPoints(body.getPos(), targetPoint);
		
		deltaAngleFront = pointVector.getDirection() - angleFront;
		deltaAngleFront = Degree::normalizeAngle(deltaAngleFront);
		deltaAngleBack = pointVector.getDirection() - angleBack;
		deltaAngleBack = Degree::normalizeAngle(deltaAngleBack);  
		
		if (fabs(deltaAngleFront) < cautionAngleFront)
		{
			setDashPower(body, targetPoint, 100, dashPower);
		   	command = new DashCommand(creator, dashPower);
			mode = CT_DASH;
		}
		else if (fabs(deltaAngleBack) < cautionAngleBack && negDashFlag)
		{
			setDashPower(body, targetPoint, -100, dashPower);
		   	command = new DashCommand(creator, dashPower);
			mode = CT_DASH;
		}
		else
		{
	   		float maxTurnAngle, deltaMaxTurnAngle;
			maxTurnAngle = BasicTurn::getTurnAngle(179, body);
			deltaMaxTurnAngle = fabs(deltaAngleFront) - maxTurnAngle;
	
			if (deltaMaxTurnAngle < cautionAngleFront / 2.00 || !negDashFlag)
			{
				command = TurnAngle(creator, deltaAngleFront, body).getCommand();
				mode = CT_TURN;
			}
			else
			{
				deltaMaxTurnAngle = deltaAngleBack - maxTurnAngle;
		
				if (deltaMaxTurnAngle < cautionAngleBack / 2.00)
				{
					command = TurnAngle(creator, deltaAngleBack, body).getCommand();
					mode = CT_TURN;
				}
				else
				{
					command = TurnAngle(creator, deltaAngleFront, body).getCommand();
					mode = CT_TURN;
				}
			}
		}
	}
}

Command *ExactGotoPoint::getCommand()
{
	return command;
}

CommandType ExactGotoPoint::getMode()
{
	return mode;
}

void PreciseGotoPoint::setDashPower(const Body &body, Point targetPoint, 
								float defaultDashPower, float &dashPower)
{
	Body tempBody;
	tempBody = body;
	
	DashCommand dashCommand(creator, 0);
	
	if (defaultDashPower >= 0)
	{
		
		float tempPower, bestDashPower = 0, bestDistance;
		bestDistance = 1000;
		for (tempPower = 0; tempPower <= 100; tempPower += 5)
		{
			tempBody = body;
	 
			float localDistance;
			dashCommand.setPower(tempPower);
	 
			tempBody.simulateByAction(&dashCommand);
			tempBody.simulateByDynamics();
		
			localDistance = hypot(tempBody.getPos().getX() - targetPoint.x, 
									tempBody.getPos().getY() - targetPoint.y);

			if (localDistance < bestDistance)
			{
				bestDistance = localDistance;
				bestDashPower = tempPower;
			}
		} 
		
		if (hypot(body.getPos().getX() - targetPoint.x,
							body.getPos().getY() - targetPoint.y) > 2)
				bestDashPower = defaultDashPower;			
	
		if (bestDashPower > body.getStaminaIncMax() && body.getStamina() < 1350)
	   		bestDashPower = body.getStaminaIncMax();
	
		dashPower = bestDashPower;
	}
	else
	{
		float tempPower, bestDashPower = 0, bestDistance;

		bestDistance = 1000;

		for (tempPower = 0; tempPower >= -100; tempPower += -5)
		{
	 	   	tempBody = body;

	 	   	float localDistance;
	 	   	dashCommand.setPower(tempPower);

	 	   	tempBody.simulateByAction(&dashCommand);
	 	   	tempBody.simulateByDynamics();
	 	   
	 	   	localDistance = hypot(tempBody.getPos().getX() - targetPoint.x, 
									tempBody.getPos().getY() - targetPoint.y);
			
			if (localDistance < bestDistance)
			{
				bestDistance = localDistance;
				bestDashPower = tempPower;
			}
		} 
		
		if (hypot(body.getPos().getX() - targetPoint.x,
							body.getPos().getY() - targetPoint.y) > 2)
				bestDashPower = defaultDashPower;			


		if (fabs(bestDashPower) > (body.getStaminaIncMax() / 2) && 
										body.getStamina() < 1400)
			bestDashPower = -(body.getStaminaIncMax() / 2);
		
		dashPower = bestDashPower;	
	}
}

PreciseGotoPoint::PreciseGotoPoint(ActionType creator, Point targetPoint, float deltaDistance, 
		const Body &body, bool negDashFlag, float minTurnAngle): BasicAction(creator)
{
	float pointDistance = hypot(targetPoint.x - body.getPos().getX(), 
								targetPoint.y - body.getPos().getY());
								
	float deltaAngleFront, deltaAngleBack, angleFront, angleBack;
	float cautionAngleBack, cautionAngleFront;
	float dashPower;
	
	command = NULL;
	mode = CT_EMPTY;

	cautionAngleBack = minTurnAngle;
	cautionAngleFront = minTurnAngle;
	
	angleFront = body.getBodyDir();
	angleBack = body.getBodyDir() - 180;
	angleBack = Degree::normalizeAngle(angleBack);
	
	if (pointDistance >= deltaDistance)
	{
		Vector pointVector;
		pointVector.setByPoints(body.getPos(), targetPoint);
		
		deltaAngleFront = pointVector.getDirection() - angleFront;
		deltaAngleFront = Degree::normalizeAngle(deltaAngleFront);
		deltaAngleBack = pointVector.getDirection() - angleBack;
		deltaAngleBack = Degree::normalizeAngle(deltaAngleBack);  
		
		if (fabs(deltaAngleFront) < cautionAngleFront)
		{
			setDashPower(body, targetPoint, 100, dashPower);
			LOG << "Meisam2005 : PreciseGTP1 : dashPower : " << dashPower;
		   	command = new DashCommand(creator, dashPower);
			mode = CT_DASH;
		}
		else if (fabs(deltaAngleBack) < cautionAngleBack && negDashFlag)
		{
			setDashPower(body, targetPoint, -100, dashPower);
			LOG << "Meisam2005 : PreciseGTP2 : dashPower : " << dashPower;
		   	command = new DashCommand(creator, dashPower);
			mode = CT_DASH;
		}
		else
		{
	   		float maxTurnAngle, deltaMaxTurnAngle;
			maxTurnAngle = BasicTurn::getTurnAngle(179, body);
			deltaMaxTurnAngle = fabs(deltaAngleFront) - maxTurnAngle;
	
			if (deltaMaxTurnAngle < cautionAngleFront / 2.00 || !negDashFlag)
			{
				command = TurnAngle(creator, deltaAngleFront, body).getCommand();
				mode = CT_TURN;
			}
			else
			{
				deltaMaxTurnAngle = deltaAngleBack - maxTurnAngle;
		
				if (deltaMaxTurnAngle < cautionAngleBack / 2.00)
				{
					command = TurnAngle(creator, deltaAngleBack, body).getCommand();
					mode = CT_TURN;
				}
				else
				{
					command = TurnAngle(creator, deltaAngleFront, body).getCommand();
					mode = CT_TURN;
				}
			}
		}
	}
}

Command *PreciseGotoPoint::getCommand()
{
	return command;
}

CommandType PreciseGotoPoint::getMode()
{
	return mode;
}

DashNormalGotoPoint::DashNormalGotoPoint(ActionType creator, const Point &target, float distance, 
		const Body &body, float maxDashPower): BasicAction(creator)
{
	float deltaAngle, validDeltaAngle;
	Vector targetVector;
	targetVector.setByPoints(body.getPos(), target);
	deltaAngle = getDeltaAngle(targetVector.getDirection(), body.getBodyDir());
	distance = max((float) EPS, distance);
	validDeltaAngle = arcTan2(distance, targetVector.getMagnitude());

	mode = CT_EMPTY;
	command = NULL;
		
	float dashPower = 100;
	if(body.getStamina() < 1300)
		dashPower = body.getStaminaIncMax();	
	if(dashPower > maxDashPower)
		dashPower = maxDashPower;

	if (fabs(deltaAngle) <= 6/*GOTOPOINTMINDELTAANGLE*/ || fabs(deltaAngle) <= validDeltaAngle)
	{
		if (targetVector.getMagnitude() > distance)
		{
			command = new DashCommand(creator, dashPower);
			mode = CT_DASH;
		}
		else
		{
			command = TurnToPoint(creator, target, body).getCommand();
			mode = CT_TURN;
		}
	}
	else
	{
		command = TurnToPoint(creator, target, body).getCommand();
		mode = CT_TURN;
	}
}


DashNormalGotoPoint::DashNormalGotoPoint(ActionType creator, const Point &target, float distance, 
		const Body &body, float idealBodyDir, float maxDashPower): BasicAction(creator)
{
	float deltaAngle, validDeltaAngle;
	Vector targetVector;
	targetVector.setByPoints(body.getPos(), target);
	deltaAngle = getDeltaAngle(targetVector.getDirection(), body.getBodyDir());
	distance = max((float) EPS, distance);
	validDeltaAngle = arcTan2(distance, targetVector.getMagnitude());

	mode = CT_EMPTY;
	command = NULL;
		
	float dashPower = 100;
	if(body.getStamina() < 2500)
		dashPower = 80;
	if(body.getStamina() < 2000)
		dashPower = 60;
	if(body.getStamina() < 1500)
		dashPower = body.getStaminaIncMax();	
	if(dashPower > maxDashPower)
		dashPower = maxDashPower;

	if (fabs(deltaAngle) <= 6/*GOTOPOINTMINDELTAANGLE*/ || fabs(deltaAngle) <= validDeltaAngle)
	{
		if (targetVector.getMagnitude() > distance)
		{
			command = new DashCommand(creator, dashPower);
			mode = CT_DASH;
		}
		else
		{
			command = TurnToAngle(creator, idealBodyDir, body).getCommand();
			mode = CT_TURN;
		}
	}
	else
	{
		command = TurnToPoint(creator, target, body).getCommand();
		mode = CT_TURN;
	}
	
	if (maxDashPower == 0)
	{
		if (command == NULL)
			command = TurnToAngle(creator, idealBodyDir, body).getCommand();
		else
		{
			delete command;
			command = TurnToAngle(creator, idealBodyDir, body).getCommand();
		}	
	}
}


Command *DashNormalGotoPoint::getCommand()
{
	return command;
}

CommandType DashNormalGotoPoint::getMode()
{
	return mode;
}


/*
DashFarNormalGotoPoint::DashFarNormalGotoPoint(const Point &target, 
	float distance, Body *body, GotoPointMode mode): BasicAction(), 
	target(target), distance(distance), body(body), mode(mode)
{
}

Command *DashFarNormalGotoPoint::getCommand()
{
	return NULL;
}
*/


/*
DashClear::DashClear(Ball *ball, Body *body, bool isOpp): BasicAction(), 
	ball(ball), body(body), isOpp(isOpp)
{
	Point kickPoint;
	float delta;
	Vector goalVector;
	
	LogMaker.Log("Clear.");

	if (isOpp)
	{
		delta = 2.5;
		if (body.getPosition()->getX() > 0)
			delta = 2.25;
		else if (body.getPosition()->getX() > 27)
			delta = 2;
//	kickPoint.X = body.getPosition()->getX() + Delta; ////////////////TOF Chasbide
//	kickPoint.Y = body.getPosition()->getY();
		basicActions.Kick.KickToPointVelocity((Point){49,0},delta,body,ball,RSC);
	}
	else
	{
		if(ball.getVelocity()->getMagnidute() > 2)
		{
			if(ball.getAbsVec()->getMagnidute() > .8)
			{
				if(body.getPosition()->getY() < 0)
				{
					kickPoint.X = 0.00; ////////////////TOF Chasbide
					kickPoint.Y = 9.00;
					BasicActions.Kick.kickToPointMaxVelocity(kickPoint,body,ball,RSC);
				}
				else
				{
					kickPoint.X = 0.00; ////////////////TOF Chasbide
					kickPoint.Y = -9.00;
					BasicActions.Kick.kickToPointMaxVelocity(kickPoint,body,ball,RSC);
				}	
			}
			else
			{
				kickPoint.X = body.getPosition()->getX() + 42; ////////////////TOF Chasbide
				kickPoint.Y = body.getPosition()->getY();
				BasicActions.Kick.kickToPointMaxVelocity(kickPoint,body,ball,RSC);
			}
		}
		else if(ball.getVelocity()->getMagnidute() > .9)
		{
			Vector ballVector;
			ballVector.setByPoints(body.getPosition()->getX(), body.getPosition()->getY(), ball.getPosition()->getX(), ball.getPosition()->getY());
			ballVector.setAsPolar(.45, ballVector.Angle);
			kickPoint.X = body.getPosition()->getX() + ballVector.X;
			kickPoint.Y = body.getPosition()->getY() + ballVector.Y;
			BasicActions.Kick.KickToPointArriveVelocity(kickPoint, 0, body, ball, RSC);
		}	
		else
		{
			///in ke joloye darvaze dar mavaghei, bayad be darvazeban pass bede va hamin joori be jolo shoot nakone ra bayad ba'dan bezaram
			kickPoint.X = body.getPosition()->getX() + 42; ////////////////TOF Chasbide
			kickPoint.Y = body.getPosition()->getY();
			BasicActions.Kick.KickToPointArriveVelocity(kickPoint, 0, body, ball, RSC);
		}
	}
}

Command *DashClear::getCommand()
{
	return NULL;
}
*/

FreeKickGotoPoint::FreeKickGotoPoint(ActionType creator, const Point &point, float standDirection,
		float collisionDistance, const Body &body): BasicAction(creator)
{
	Vector pointVector;
	Vector standVector;
	Vector targetVector;
	Vector goVector;
	Point targetPoint;
	float distance;
	bool intersectFlag = false;

	command = NULL;
	
#ifdef FREEKICK_GOTO_POINT_LOG
LOG << "FKGotoPoint 1targetP : " << point << endl;
#endif
#ifdef FREEKICK_GOTO_POINT_LOG
LOG << "FKGotoPoint coldist : " << collisionDistance << endl;
#endif
	standVector.setAsPolar(body.getSize() + collisionDistance + 0.1, standDirection + 180);
#ifdef FREEKICK_GOTO_POINT_LOG
LOG << "FKGotoPoint 2standV : " << standVector << endl;
#endif
	targetPoint = (Vector&)point + standVector;
#ifdef FREEKICK_GOTO_POINT_LOG
LOG << "FKGotoPoint 2targetP : " << targetPoint << endl;
#endif
	pointVector.setByPoints(body.getPos(), point);
	targetVector.setByPoints(body.getPos(), targetPoint);
#ifdef FREEKICK_GOTO_POINT_LOG
LOG << "FKGotoPoint 1targetV : " << targetVector << endl;
#endif
	
	Line targetLine;
	targetLine.setByPoints(body.getPos(), targetPoint);
	distance = targetLine.getPointDist(point);
#ifdef FREEKICK_GOTO_POINT_LOG
LOG << "FKGotoPoint distance : " << distance << endl;
#endif
    if (distance < body.getSize() + collisionDistance + 0.01 &&
		targetVector.getMagnitude() > standVector.getMagnitude() &&
		pointVector.getMagnitude() < targetVector.getMagnitude())
	{ // intersect
#ifdef FREEKICK_GOTO_POINT_LOG
LOG << "FKGotoPoint intersect " << endl;
#endif
		float angleDiff;

		intersectFlag = true;
		angleDiff = normalizeAngle(
			absoluteAngle(standVector.getDirection()) -
			absoluteAngle(180 + pointVector.getDirection()));
		standVector.setAsPolar(body.getSize() + collisionDistance + 2.0, pointVector.getDirection() + (angleDiff / 2) + 180);
		targetPoint = (Vector&)point + standVector;
		targetVector.setByPoints(body.getPos(), targetPoint);
	}
	else
	{
	}
	goVector = targetVector;
	
#ifdef FREEKICK_GOTO_POINT_LOG
LOG << "FKGotoPoint 3targetP : " << targetPoint << endl;
#endif
	ExactGotoPoint doGotoPoint(creator, targetPoint, 0.2, body);
	command = doGotoPoint.getCommand();
	
#ifdef FREEKICK_GOTO_POINT_LOG
LOG << "FKGotoPoint mode : " << doGotoPoint.getMode() << endl;
#endif
	if (command)
	{
#ifdef FREEKICK_GOTO_POINT_LOG
LOG << "FKGotoPoint command : " << command->toString() << endl;
#endif
	}
	if (intersectFlag || doGotoPoint.getMode() != CT_EMPTY)
		status = 1;
	else
		status = 0;
}

Command *FreeKickGotoPoint::getCommand()
{
	return command;
}

int FreeKickGotoPoint::getStatus() const
{
	return status;
}
/*   2           1                
       |---------------------- Y2
3   X1 |
	   |---------------------- Y1
*/// 4           5                


ForbiddenAreaGotoPoint::ForbiddenAreaGotoPoint(ActionType creator, const Point &point, float rectX1,
		float rectY1, float rectY2, const Body &body): BasicAction(creator)
{
	int sourcePlace = 0;
	int goModel = 0;

#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "ForbiddenAreaGotoPoint Rect" << endl;
#endif

	command = NULL;

#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint Point 1st=" << point <<  endl;
#endif

	rectY1 = min(rectY1, rectY2);
	rectY2 = max(rectY1, rectY2);

#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "ForbiddenAreaGotoPoint X1 = " << rectX1 << endl;
LOG << "ForbiddenAreaGotoPoint Y1 = " << rectY1 << endl;	
LOG << "ForbiddenAreaGotoPoint Y2 = " << rectY2 << endl;	
#endif

	forbiddenSourceFlag = false;
	forbiddenTargetFlag = false;
	realTargetPlace = 1;
	realTargetPoint = point;
	goAroundFlag = false;
	
	if (point.x > rectX1 &&	point.y > rectY1 && point.y < rectY2)
	{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint forbiddenTargetFlag = true" << endl;
#endif
		float minDistance;
		forbiddenTargetFlag = true;

		realTargetPoint.x = rectX1;
		minDistance = point.x - rectX1;

		if (point.y - rectY1 < minDistance)
		{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint near1" << endl;
#endif
			realTargetPoint.x = point.x;
			realTargetPoint.y = rectY1;
			minDistance = point.y - rectY1;
		}

		if (rectY2 - point.y < minDistance)
		{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint near2" << endl;
#endif
			realTargetPoint.x = point.x;
			realTargetPoint.y = rectY2;
			minDistance = rectY2 - point.y;
		}
	}

#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint Point 2nd=" << realTargetPoint <<  endl;
#endif

	if (body.getPos().getX() > rectX1 + 0.8 &&
		body.getPos().getY() > rectY1 + 0.8 &&
	   	body.getPos().getY() < rectY2 - 0.8)
	{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint forbiddenSourceFlag = true" << endl;
#endif
		forbiddenSourceFlag = true;
		float minDistance;

		realTargetPoint.x = rectX1;
		minDistance = body.getPos().getX() - rectX1;

		if (body.getPos().getY() - rectY1 < minDistance)
		{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint fsource near1" << endl;
#endif
			realTargetPoint.x = body.getPos().getX();
			realTargetPoint.y = rectY1;
			minDistance = body.getPos().getY() - rectY1;
		}

		if (rectY2 - body.getPos().getY() < minDistance)
		{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint fsource near2" << endl;
#endif
			realTargetPoint.x = body.getPos().getX();
			realTargetPoint.y = rectY2;
			minDistance = rectY2 - body.getPos().getY();
		}
		DashNormalGotoPoint doGotoPoint(creator, realTargetPoint, 1.2, body, 90);
		command = doGotoPoint.getCommand();
		if (doGotoPoint.getMode() == CT_EMPTY)
			status = 0;
		else
			status = 1;	
		return;
		/*
		command = new EmptyCommand();
		status = 0;
		return;
		*/
	}
	
	if (realTargetPoint.x > rectX1 && realTargetPoint.y >= rectY2)
		realTargetPlace = 1;
	if (realTargetPoint.x <= rectX1 && realTargetPoint.y >= rectY2)
		realTargetPlace = 2;
	if (realTargetPoint.x <= rectX1 && realTargetPoint.y < rectY2 &&
		realTargetPoint.y > rectY1)
		realTargetPlace = 3;
	if (realTargetPoint.x <= rectX1 && realTargetPoint.y <= rectY1)
		realTargetPlace = 4;
	if (realTargetPoint.x > rectX1 && realTargetPoint.y <= rectY1)
		realTargetPlace = 5;

	if (body.getPos().getX() > rectX1 && body.getPos().getY() >= rectY2)
		sourcePlace = 1;
	if (body.getPos().getX() <= rectX1 && body.getPos().getY() >= rectY2)
		sourcePlace = 2;
	if (body.getPos().getX() <= rectX1 && body.getPos().getY() < rectY2 &&
		body.getPos().getY() > rectY1)
		sourcePlace = 3;
	if (body.getPos().getX() <= rectX1 && body.getPos().getY() <= rectY1)
		sourcePlace = 4;
	if (body.getPos().getX() > rectX1 && body.getPos().getY() <= rectY1)
		sourcePlace = 5;

#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint realTargetPlace = " << realTargetPlace << endl;
LOG << "FAGotoPoint sourcePlace" << sourcePlace <<  endl;
#endif
		
	if (sourcePlace == realTargetPlace)
		goModel = 0;
	else if (abs(sourcePlace - realTargetPlace) == 1)
		goModel = 0;
	else if ((sourcePlace == 1 && realTargetPlace == 3) || 
			 (sourcePlace == 1 && realTargetPlace == 4) ||
			 (sourcePlace == 1 && realTargetPlace == 5))
		goModel = 1;
	else if (sourcePlace == 2 && realTargetPlace == 4)
		goModel = 0;
	else if (sourcePlace == 2 && realTargetPlace == 5)
		goModel = 2;
	else if (sourcePlace == 3 && realTargetPlace == 1)
		goModel = 1;
	else if (sourcePlace == 3 && realTargetPlace == 5)
		goModel = 2;
	else if (sourcePlace == 4 && realTargetPlace == 1)
		goModel = 1;
	else if (sourcePlace == 4 && realTargetPlace == 2)
		goModel = 0;
	else if ((sourcePlace == 5 && realTargetPlace == 1) || 
			 (sourcePlace == 5 && realTargetPlace == 2) ||
			 (sourcePlace == 5 && realTargetPlace == 3))
		goModel = 2;

#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint goModel = " << goModel << endl;
#endif
	
	command = NULL;

	if (goModel == 0)
	{
		DashNormalGotoPoint doGotoPoint(creator, realTargetPoint, 1.4, body, 90);
		command = doGotoPoint.getCommand();
		if (doGotoPoint.getMode() == CT_EMPTY)
			status = 0;
		else
			status = 1;	
	}
	if (goModel == 1)
	{
		goAroundFlag = true;
		DashNormalGotoPoint doGotoPoint(creator, Point(rectX1 - 1.5,rectY2 + 1.5), 1,
				body, 90);
		command = doGotoPoint.getCommand();
		if (doGotoPoint.getMode() == CT_EMPTY)
			status = 0;
		else
			status = 1;	
	}
	if (goModel == 2)
	{
		goAroundFlag = true;
		DashNormalGotoPoint doGotoPoint(creator, Point(rectX1 - 1.5,rectY1 - 1.5), 1,
				body, 90);
		command = doGotoPoint.getCommand();
		if (doGotoPoint.getMode() == CT_EMPTY)
			status = 0;
		else
			status = 1;	
	}
}

ForbiddenAreaGotoPoint::ForbiddenAreaGotoPoint(ActionType creator, const Point &point, Point &rectNW,
		 Point &rectSE, const Body &body): BasicAction(creator)
{
	ForbiddenAreaGotoPoint(creator, rectNW.x, rectNW.y, rectSE.y, body);
}

ForbiddenAreaGotoPoint::ForbiddenAreaGotoPoint(ActionType creator, const Point &point,
		 Point circleCenter, float circleRadius, const Body &body,
		 bool forbiddenX): BasicAction(creator)
{
	Vector sourceCenter;
	Vector targetCenter;
	Vector sourceTarget;
	Vector moveVector;
	float formulaA = 0;
	bool intersectFlag;	
	forbiddenSourceFlag = false;
	forbiddenTargetFlag = false;
	realTargetPlace = 0;
	realTargetPoint = point;
	intersectFlag = false;
	goAroundFlag = false;

	command = NULL;
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "ForbiddenAreaGotoPoint Circle" << endl;
#endif
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint Point = " << point << endl;	
#endif
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint Center = " << circleCenter << endl;	
#endif
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint Radius = " << circleRadius << endl;	
#endif
	targetCenter.setByPoints(point, circleCenter);
	sourceCenter.setByPoints(body.getPos(), circleCenter);
	if (forbiddenX && point.x > circleCenter.x - 1.5)
	{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint forbiddenTargetX = true" << endl;
#endif
		forbiddenTargetFlag = true;
		realTargetPoint.y = point.y;
		realTargetPoint.x = circleCenter.x - 1.5;
		targetCenter.setByPoints(realTargetPoint, circleCenter);
		
	}
	if (targetCenter.getMagnitude() < circleRadius)
	{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint forbiddenTarget = true" << endl;
#endif
		forbiddenTargetFlag = true;
		moveVector = targetCenter;
		moveVector *= -1;
		moveVector.setAsPolar(circleRadius + 0.5, moveVector.getDirection());
		realTargetPoint = (Vector)circleCenter + moveVector;
		targetCenter.setByPoints(realTargetPoint, circleCenter);
	}
	sourceTarget.setByPoints(body.getPos(), realTargetPoint);

	if (body.getDistance(circleCenter) < circleRadius - 1.3)
	{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint forbiddenSource = true" << endl;
#endif
		forbiddenSourceFlag = true;
		targetCenter.setByPoints(body.getPos(), circleCenter);
		moveVector = targetCenter;
		moveVector *= -1;
		moveVector.setAsPolar(circleRadius + 0.5, moveVector.getDirection());
		realTargetPoint = (Vector)circleCenter + moveVector;

		DashNormalGotoPoint doGotoPoint(creator, realTargetPoint, 1.5, body, 90);
		command = doGotoPoint.getCommand();
		if (doGotoPoint.getMode() == CT_EMPTY)
			status = 0;
		else
			status = 1;	
		return;
		/*
		command = new EmptyCommand();
		status = 0;
		return;
		*/
	}

	command = NULL;
	
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint newPoint = " << realTargetPoint << endl;	
#endif

	Line goLine;
	goLine.setByPoints(body.getPos(), realTargetPoint);
	formulaA = goLine.getPointDist(circleCenter);
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint formulaA = " << formulaA << endl;	
#endif
	if (formulaA < circleRadius - 0.07 &&
	   	sourceTarget.getMagnitude() > sourceCenter.getMagnitude())
	{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint <radius " << endl;	
#endif
		Point tangence1, tangence2;
		float tangenceStandDistance;
		Point goPoint;
		bool tangencePossible1, tangencePossible2;
		Vector tangenceVector1;
		Vector tangenceVector2;
		bool directDash1, directDash2;

		float centerTargetAngle;
		float centerSourceAngle;
		float middleAngle;

		goAroundFlag = true;
		centerSourceAngle = normalizeAngle(sourceCenter.getDirection() + 180);
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint centerSourceAngle=" << centerSourceAngle << endl;	
#endif
		centerTargetAngle = normalizeAngle(targetCenter.getDirection() + 180);
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint centerTargetAngle=" << centerTargetAngle << endl;
#endif
		middleAngle = (centerTargetAngle - centerSourceAngle) / 2;
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint middleAngle=" << middleAngle << endl;	
#endif
		
		directDash1 = directDash2 = true;
		intersectFlag = true;
		tangencePossible1 = true;
		tangencePossible2 = true;
		tangenceStandDistance = Basics::reRate(sourceCenter.getMagnitude() - circleRadius,
				0, 10, 0, 10, RRM_REVERSE);
		moveVector.setAsPolar(circleRadius + tangenceStandDistance,
				centerSourceAngle + middleAngle);
		tangence1 = (Vector)circleCenter + moveVector;
		moveVector.setAsPolar(circleRadius + tangenceStandDistance,
				centerSourceAngle + middleAngle + 180);
		tangence2 = (Vector)circleCenter + moveVector;
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint tp1" << tangence1 << endl;
#endif
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint tp2" << tangence2 << endl;
#endif
		if (fabs(tangence1.x) > 56.5 || fabs(tangence1.y) > 38 ||
				(forbiddenX && tangence1.x > circleCenter.x))
			tangencePossible1 = false;
		if (fabs(tangence2.x) > 56.5 || fabs(tangence2.y) > 38 ||
				(forbiddenX && tangence2.x > circleCenter.x))
			tangencePossible2 = false;
		tangenceVector1.setByPoints(body.getPos(), tangence1);
		tangenceVector2.setByPoints(body.getPos(), tangence2);
/*
		if (tangenceVector1.getMagnitude() < 0.45)
		{
			tangencePossible1 = true;
			moveVector.setAsPolar(circleRadius + 2.8,
					targetCenter.getDirection() + 45);
			tangence1 = (Vector)circleCenter + moveVector;
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPointnew tp1" << tangence1 << endl;
#endif
			if (fabs(tangence1.x) > 56.5 || fabs(tangence1.y) > 38)
				tangencePossible1 = false;
			tangenceVector1.setByPoints(body.getPos(),tangence1);
		}
		
		if (tangenceVector2.getMagnitude() < 0.45)
		{
			tangencePossible2 = true;
			moveVector.setAsPolar(circleRadius + 2.8,
					targetCenter.getDirection() - 45);
			tangence2 = (Vector)circleCenter + moveVector;
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPointnew tp2" << tangence2 << endl;
#endif
			if (fabs(tangence2.x) > 56.5 || fabs(tangence2.y) > 38)
				tangencePossible2 = false;
			tangenceVector2.setByPoints(body.getPos(),tangence2);
		}
*/
		if (!tangencePossible1)
		{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint select2 mode 1" << endl;
#endif
			goPoint = tangence2;
		}
		else if (!tangencePossible2)
		{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint select1 mode 1" << endl;
#endif
			goPoint = tangence1;
		}
		else if (tangenceVector1.getMagnitude() < tangenceVector2.getMagnitude()		-1 &&
				tangencePossible1)
		{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint select1 mode 2" << endl;
#endif
			goPoint = tangence1;
		}
		else if (tangenceVector2.getMagnitude() < tangenceVector1.getMagnitude()		-1 &&
				tangencePossible2)
		{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint select2 mode 2" << endl;
#endif
			goPoint = tangence2;
		}
		else
		{
			DashNormalGotoPoint checkGotoPoint1 (creator, tangence1, 0.9, body);
			DashNormalGotoPoint checkGotoPoint2 (creator, tangence2, 0.9, body);
			delete checkGotoPoint1.getCommand();
			delete checkGotoPoint2.getCommand();
			if (checkGotoPoint1.getMode() == CT_TURN)
				directDash1 = false;
			else
				directDash1 = true;
			if (checkGotoPoint2.getMode() == CT_TURN)
				directDash2 = false;
			else
				directDash2 = true;
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint directdashes" << directDash1 << " " << directDash2 << endl;
#endif
			if (tangencePossible1 && directDash1 && !directDash2)
			{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint select1 mode 3" << endl;
#endif
				goPoint = tangence1;
			}
			else if (tangencePossible2 && directDash2 && !directDash1)
			{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint select2 mode 3" << endl;
#endif
				goPoint = tangence2;
			}
			else if (fabs(tangence1.y) < fabs(tangence2.y))
			{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint select1 mode 4" << endl;
#endif
				goPoint = tangence1;
			}
			else
			{
#ifdef FORBIDDENAREA_GOTO_POINT_LOG
LOG << "FAGotoPoint select2 mode 4" << endl;
#endif
				goPoint = tangence2;
			}
		}
		DashNormalGotoPoint doGotoPoint(creator, goPoint, 1.1, body, 90);
		command = doGotoPoint.getCommand();
		if (doGotoPoint.getMode() == CT_EMPTY)
			status = 0;
		else
			status = 1;	
	}
	else
	{
		DashNormalGotoPoint doGotoPoint(creator, realTargetPoint, 1.4, body, 90);
		command = doGotoPoint.getCommand();
		if (doGotoPoint.getMode() == CT_EMPTY)
			status = 0;
		else
			status = 1;	
	}
}

Command *ForbiddenAreaGotoPoint::getCommand()
{
	return command;
}

bool ForbiddenAreaGotoPoint::isForbiddenSource() const
{
	return forbiddenSourceFlag;
}

bool ForbiddenAreaGotoPoint::isForbiddenTarget() const
{
	return forbiddenTargetFlag;
}

int ForbiddenAreaGotoPoint::getRealTargetPlace() const
{
	return realTargetPlace;
}

Point ForbiddenAreaGotoPoint::getRealTargetPoint() const
{
	return realTargetPoint;
}

bool ForbiddenAreaGotoPoint::isClockwise() const
{
	return clockwiseFlag;
}

int ForbiddenAreaGotoPoint::getStatus() const
{
	return status;
}
