/*
 *  Copyright 2002-2005, Mersad Team, Allameh Helli High School (NODET).
 *
 *  This program is free software, you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Library General Public License for more details.
 *
 *  This file is created by: Arash Rahimi
 *
 *  Released on Monday 1 August 2005, 10 Mordad 1384 by Mersad RoboCup Team.
 *  For more information please read README file.
*/

#ifndef _STATELESS_PLAN_H_
#define _STATELESS_PLAN_H_

#include <vector>

#include <AdvancedAction.h>
#include <Plan.h>
#include <GroupWorks.h>
#include <Positioning.h>
#include <SyncedAgent.h>
#include <SayDecision.h>
#include <BodyDecision.h>
#include <HeadDecision.h>
#include <Positioning.h>
#include <PointToDecision.h>
#include <PenaltyDecision.h>
#include <NonPlayOnDecision.h>
#include <GoaliePlayOnDecision.h>
#include <WithBallPlayOnDecision.h>
#include <WithoutBallPlayOnDecision.h>
#include <HeadDecision.h>

#ifndef NULL
#	define NULL 0
#endif // NULL

class StatelessPlan: public Plan
{
public:
	StatelessPlan(const WorldModel *wm, Form &form, const Library &library);
	virtual ~StatelessPlan();

	virtual bool isFinished();
	virtual double successRate();

	virtual void decide();

	virtual void senseBody(const SExpression &exp);
	virtual void see(const SExpression &exp);
	virtual void hear(const SExpression &exp);

protected:
	Command *sayCycleCommand;
	Command *bodyCycleCommand;
	Command *headCycleCommand;
	Command *pointToCycleCommand;
	Command *attentionCycleCommand;
	Command *changeViewCycleCommand;

	NonPlayOnDecision nonPlayOnDecision;
	PenaltyDecision penaltyDecision;
	GoaliePlayOnDecision goaliePlayOnDecision;
	WithBallPlayOnDecision withBallPlayOnDecision;
	WithoutBallPlayOnDecision withoutBallPlayOnDecision;

	bool lastWithBallDecideValidFlag;

	void setAdvancedActions();
	void checkForLastWithBallDecideValid();

	// Getting functions
	bool isLastWithBallDecideValid() const;

	// Setting functions
	void setLastWithBallDecideValid(bool withBallDecide);

};

#endif //_STATELESS_PLAN_H_
